// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

#include "stdafx.h"

#include "degenerate_na.h"

#include <ctype.h>
#include <vector>

using namespace std;

list<DNA3Seq> expand_nucleic_acid(const string &m_seq)
{
	const unsigned int len = m_seq.size();
	unsigned int i, j;

	list<DNA3Seq> seq_list;
	
	vector<string> degen_vec( len );
	
	for(i = 0;i < len;i++){
		switch( toupper(m_seq[i]) ){
			case 'A':
				degen_vec[i] = "A";
				break;
			case 'T':
				degen_vec[i] = "T";
				break;
			case 'G':
				degen_vec[i] = "G";
				break;
			case 'C':
				degen_vec[i] = "C";
				break;
			case 'M':
				degen_vec[i] = "AC";
				break;
			case 'R':
				degen_vec[i] = "GA";
				break;
			case 'S':
				degen_vec[i] = "GC";
				break;
			case 'V':
				degen_vec[i] = "GCA";
				break;
			case 'W':
				degen_vec[i] = "AT";
				break;
			case 'Y':
				degen_vec[i] = "TC";
				break;
			case 'H':
				degen_vec[i] = "ACT";
				break;
			case 'K':
				degen_vec[i] = "GT";
				break;
			case 'D':
				degen_vec[i] = "GAT";
				break;
			case 'B':
				degen_vec[i] = "GTC";
				break;
			case 'N':
				degen_vec[i] = "ATGC";
				break;
			default:
				throw "Unknown base!";
		};
	}
	
	unsigned int num_seq = 1;
	
	vector<string::const_iterator> iter_vec(len);
	
	for(i = 0;i < len;i++){
		iter_vec[i] = degen_vec[i].begin();
		
		num_seq *= degen_vec[i].size();
	}
	
	for(i = 0;i < num_seq;i++){
		DNA3Seq tmp(len);
		DNA3Seq::iterator tmp_iter = tmp.begin();

		for(j = 0;j < len;j++, tmp_iter++){
			tmp_iter.c_base(*iter_vec[j]);
		}
		
		seq_list.push_back(tmp);
		
		for(j = 0;j < len;j++){
			iter_vec[j]++;
			
			if( iter_vec[j] == degen_vec[j].end() ){
				// Wrap around
				iter_vec[j] = degen_vec[j].begin();
			}
			else{
				break;
			}
		}
	}
	
	return seq_list;
}
