// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

// Genomorama for X11
// version 1.0 (3/24/04)
// 
// This file contains all X11/Motif dependent code! For easier compiles and 
// better organization, this file sould be split up.
//
// J. D. Gans (jgans@lanl.gov)
// Bioscience division, B5
// Los Alamos National Lab
//


#include <stdlib.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeB.h>
#include <Xm/Frame.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>
#include <Xm/MessageB.h>
#include <Xm/LabelG.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/PanedW.h>
#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/TextF.h>
#include <Xm/Scale.h>
#include <Xm/List.h>
#include <Xm/ScrollBar.h>

#include <Xm/Protocols.h>
#include <Xm/AtomMgr.h>

#include <X11/keysym.h>
#include <X11/cursorfont.h>

#include <GL/GLwMDrawA.h>  // Motif OpenGL drawing area widget 
#include <GL/glx.h>

#include "sovLayerUtil.h"
#include "font_file.h"
#include "ncbi_util.h"

// The icon in XPM format
#include "genomorama.bm"

#include <iostream>
#include <sstream>

#include <sys/time.h>

// Boolean is used by the NCBI libraries -- avoid a namespace
// collision!
typedef Boolean XBool;

/////////////////////////////////////////////////
#include "genome.h"
#include "primer.h"
/////////////////////////////////////////////////

// An email address for users to ask for help ...
#define	EMAIL_ADDRESS		"jgans@lanl.gov"

// Offset the height of the canvas to work around GLX/X strangeness
#define	HEIGHT_OFFSET		2

using namespace std;

static String fallbackResources[] = {
	"*title: Genomorama v" GV_VERSION_MAJOR "." GV_VERSION_MINOR,
	"*glxarea*width: 600",
	"*glxarea*height: 300",
	
	"*XmText.translations: #override\\n"
        "Ctrl~Alt~Meta<KeyPress>v: pasteclipboard()\\n"
        "Ctrl~Alt~Meta<KeyPress>c: copyclipboard()\\n"
        "Ctrl~Alt~Meta<KeyPress>x: cutclipboard()\\n",
    	"*XmTextField.translations: #override\\n"
        "Ctrl~Alt~Meta<KeyPress>v: pasteclipboard()\\n"
        "Ctrl~Alt~Meta<KeyPress>c: copyclipboard()\\n"
        "Ctrl~Alt~Meta<KeyPress>x: cutclipboard()\\n",
	
	"*file.open.accelerator: Ctrl<Key>o",
	"*file.open.acceleratorText: Ctrl+O",
	"*file.exit.accelerator: Ctrl<Key>q",
	"*file.exit.acceleratorText: Ctrl+Q",
	"*edit.select_all.accelerator: Ctrl<Key>a",
	"*edit.select_all.acceleratorText: Ctrl+A",
	"*edit.find.accelerator: Ctrl<Key>f",
	"*edit.find.acceleratorText: Ctrl+F",
	"*edit.find_again.accelerator: Ctrl<Key>g",
	"*edit.find_again.acceleratorText: Ctrl+G",
//	"*edit.hybridize.accelerator: Ctrl<Key>h",
//	"*edit.hybridize.acceleratorText: Ctrl+H",
	"*view.Magnify.accelerator: Ctrl<Key>m",
	"*view.Magnify.acceleratorText: Ctrl+M",
	"*fit_to_window.accelerator: Ctrl<Key>r",
	"*fit_to_window.acceleratorText: Ctrl+R",
	"*move.up.accelerator: Shift<Key>KP_Up",
	"*move.up.acceleratorText: Shift+Up",
	"*move.down.accelerator: Shift<Key>KP_Down",
	"*move.down.acceleratorText: Shift+Down",
	//"*show.one_more.accelerator: <Key>+",
	"*show.one_more.acceleratorText: +",
	"*show.one_less.accelerator: <Key>-",
	//"*show.one_less.acceleratorText: -",
	"*genome.complement.accelerator: Ctrl<Key>i",
	"*genome.complement.acceleratorText: Ctrl+I",
	"*genome.edit_annotation.accelerator: Ctrl<Key>e",
	"*genome.edit_annotation.acceleratorText: Ctrl+E",
  	NULL
};

#define	HSCROLL_MIN		0
#define	HSCROLL_MAX		100
#define	HSCROLL_CENTER		50
#define	HSCROLL_DELTA		1

#define	VSCROLL_MIN		0
#define	VSCROLL_MAX		100
#define	VSCROLL_CENTER		50
#define	VSCROLL_DELTA		1

// More Global variables than you can shake a stick at!
XtAppContext app;
Display *dpy;
int doubleBuffer = 1, 
    madeCurrent = 0;

// The various sub-windows that a user can open (and close)
// These should all be turned into classes!
Widget file_dialog = NULL;
Widget color_dialog = NULL;
Widget info_dialog = NULL;
Widget find_dialog = NULL;
Widget hybrid_dialog = NULL;
Widget padlock_dialog = NULL;
Widget options_dialog = NULL;
Widget download_dialog = NULL;
Widget primer_dialog = NULL;
Widget export_genome_dialog = NULL;
Widget annotation_dialog = NULL;

// Main window and dialog GUI elements
Widget 	toplevel, 
	 	mainw, 
		menubar, 
		menupane,
		submenupane,
		subsubmenupane,
		btn, 
		cascade, 
		frame, 
		glxarea, 
		dialog, 
		popup,
		status_bar,
		status_bar_info,
		status_bar_base,
		info_dialog_text,
		find_dialog_search_text,
		find_dialog_results_text,
		options_dialog_font_scaling,
		options_dialog_gc_window,
		options_dialog_tm_window,
		options_dialog_salt,
		options_dialog_strand,
		hybrid_dialog_strand_text,
		hybrid_dialog_salt_text,
		hybrid_dialog_tm_text,
		hybrid_dialog_exact_text,
		hybrid_dialog_min_amplicon_len_text,
		hybrid_dialog_max_amplicon_len_text,
		hybrid_dialog_primer1_text,
		hybrid_dialog_primer2_text,
		hybrid_dialog_probe_text,
		hybrid_dialog_results_text,
		padlock_dialog_strand_text,
		padlock_dialog_salt_text,
		padlock_dialog_tm_text,
		padlock_dialog_exact_text,
		padlock_dialog_primer1_text,
		padlock_dialog_primer2_text,
		padlock_dialog_results_text,
		download_dialog_search_text,
		download_dialog_doc_list,
		size_centric_toggle,
		gene_centric_toggle,
		primer_dialog_primer_length_text,
		primer_dialog_min_tm_text,
		primer_dialog_hairpin_tm_text,
		primer_dialog_salt_text,
		primer_dialog_strand_text,
		primer_dialog_3_GC_toggle,
		primer_dialog_GC_5_toggle,
		primer_dialog_run_toggle,
		primer_dialog_run_length_text,
		primer_dialog_GC_toggle,
		primer_dialog_min_GC_text,
		primer_dialog_max_GC_text,
		primer_dialog_3_T_toggle,
		export_genome_gene_toggle,
		export_genome_pseudo_gene_toggle,
		export_genome_CDS_toggle,
		export_genome_RNA_toggle,
		export_genome_tRNA_toggle,
		export_genome_misc_toggle,
		export_genome_intergenic_toggle,
		export_genome_primer_toggle,
		export_genome_user_toggle,
		export_genome_fasta_genome_radio,
		export_genome_fasta_dna_radio,
		export_genome_fasta_aa_radio,
		export_genome_fasta_gbk_radio,
		export_genome_fasta_ptt_radio,
		annot_dialog_range_text,
		annot_dialog_locus_text,
		annot_dialog_locus_tag_text,
		annot_dialog_product_text,
		annot_dialog_note_text,
		annot_dialog_ec_text,
		annot_dialog_gi_text,
		annot_dialog_accession_text,
		annot_dialog_type_option,
		annot_dialog_strand_option,
		vertical_scroll_bar,
		horizontal_scroll_bar;

Pixmap icon = 0;
Pixmap icon_mask = 0;

unsigned char color_dialog_rgb[4]; //rgba

// What color is the color dialog modifying?
XtPointer color_callData = NULL;

// Annotation dialog parameters
std::string annot_range_text = "";
std::string annot_locus_text = "";
std::string annot_locus_tag_text = "";
std::string annot_product_text = "";
std::string annot_note_text = "";
std::string annot_ec_text = "";
std::string annot_gi_text = "";
std::string annot_accession_text = "";
int annot_strand = Seq_strand_plus;
int annot_type = GeneAnnotation::CDS;
selected_annotation_info edit_annot;

// Find dialog parameters
std::string search_text = "";
std::string search_results = "";
int search_key = GenomePage::SEARCH_KEY_NAME;

// Download diaglog parameters
int download_key = ENTREZ_GENOME_ACCESSION;
std::list< std::pair<unsigned int, std::string> > docsum_list;
std::string download_search_text = "";

// Hybridization diaglog parameters
#ifdef _USE_PRIMER3
std::string hybrid_strand_text = "5.0e-8";
std::string hybrid_salt_text = "0.05";
#else // SANTA_LUCIA parameters
std::string hybrid_strand_text = "1.0e-7";
std::string hybrid_salt_text = "0.2";
#endif // _USE_PRIMER3

std::string hybrid_tm_text = "50.0";
std::string hybrid_exact_text = "3";
std::string hybrid_min_amplicon_len_text = "0";
std::string hybrid_max_amplicon_len_text = "2000";
std::string hybrid_primer1_text = "";
std::string hybrid_primer2_text = "";
std::string hybrid_probe_text = "";
std::string hybrid_results_text = "";

// Padlock PCR hybridization diaglog parameters
#ifdef _USE_PRIMER3
std::string padlock_strand_text = "5.0e-8";
std::string padlock_salt_text = "0.05";
#else // SANTA_LUCIA parameters
std::string padlock_strand_text = "1.0e-7";
std::string padlock_salt_text = "0.2";
#endif // _USE_PRIMER3

std::string padlock_tm_text = "50.0";
std::string padlock_exact_text = "3";
std::string padlock_primer1_text = "";
std::string padlock_primer2_text = "";
std::string padlock_results_text = "";

// Primer diaglog parameters
std::string primer_len_text = "20";
std::string primer_tm_text = "50";
std::string primer_hairpin_tm_text = "10";
#ifdef _USE_PRIMER3
std::string primer_salt_text = "0.05";
std::string primer_strand_text = "5.0e-8";
#else // SANTA_LUCIA parameters
std::string primer_salt_text = "0.2";
std::string primer_strand_text = "1.0e-7";
#endif // _USE_PRIMER3

std::string primer_run_len_text = "4";
std::string primer_min_gc_text = "45";
std::string primer_max_gc_text = "55";

bool primer_3_GC_toggle = true;
bool primer_GC_5_toggle = true;
bool primer_run_toggle = true;
bool primer_GC_toggle = true;
bool primer_3_T_toggle = true;

XtTranslations trans = NULL;
Window glxwin;
GLXContext cx;
XVisualInfo *vi = NULL;
Arg args[10];
XFontStruct *fontInfo;
Dimension viewWidth, viewHeight;
Visual *overlayVisual = NULL;
int overlayDepth;
Colormap overlayColormap;
bool redisplayPending = false;

int render_opt = 0;
GenomePage *ptrGP = NULL;

// local functions
void quit(Widget w, XtPointer data, XtPointer callData);
void WaitCursor(Widget w);
void ArrowCursor(Widget w);

void AddMenuItem(Widget parent, char *name, char *label,
	char mnemonic, XtCallbackProc callback);

void AddMenuSeperator(Widget parent);
	
void OnFileExportPs(Widget w, XtPointer data, XtPointer callData);
void OnFileExportPsOk(Widget w, XtPointer data, XtPointer callData);

void OnFileExportGif(Widget w, XtPointer data, XtPointer callData);
void OnFileExportGifOk(Widget w, XtPointer data, XtPointer callData);

void OnFileExportGenomes(Widget w, XtPointer data, XtPointer callData);

void OnFileOpen(Widget w, XtPointer data, XtPointer callData);
void OnFileOpenOk(Widget w, XtPointer text, XtPointer callData);
void OnFileOpenCancel(Widget w, XtPointer data, XtPointer callData);

void OnFileLoadGenomePlot(Widget w, XtPointer data, XtPointer callData);
void OnFileLoadGenomePlotOk(Widget w, XtPointer text, XtPointer callData);
void OnFileLoadGenomePlotCancel(Widget w, XtPointer data, XtPointer callData);
	
void OnEditDelete(Widget w, XtPointer data, XtPointer callData);
void OnEditSelectAll(Widget w, XtPointer data, XtPointer callData);
void OnEditFind(Widget w, XtPointer data, XtPointer callData);
void OnEditFindAgain(Widget w, XtPointer data, XtPointer callData);
void OnEditHybridize(Widget w, XtPointer data, XtPointer callData);
void OnEditHybridizePadlock(Widget w, XtPointer data, XtPointer callData);
void OnEditPrimer(Widget w, XtPointer data, XtPointer callData);
void OnEditPrimerDelete(Widget w, XtPointer data, XtPointer callData);

void OnViewOptions(Widget w, XtPointer data, XtPointer callData);
void OnViewSizeCentric(Widget w, XtPointer data, XtPointer callData);
void OnViewGeneCentric(Widget w, XtPointer data, XtPointer callData);
void OnViewMomentum(Widget w, XtPointer data, XtPointer callData);
void OnViewMagnify(Widget w, XtPointer data, XtPointer callData);
void OnViewFitToWindow(Widget w, XtPointer data, XtPointer callData);

void OnGenomeHide(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationImport(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationImportOk(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationEdit(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationEditApply(Widget w, XtPointer data, XtPointer callData);
bool OnGenomeAnnotationUpdate();
void AnnotationDialogClose(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationDelete(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAnnotationHide(Widget w, XtPointer data, XtPointer callData);
void OnGenomeShowAll(Widget w, XtPointer data, XtPointer callData);
void OnGenomeShowAnnotation(Widget w, XtPointer data, XtPointer callData);
void OnGenomeGC(Widget w, XtPointer data, XtPointer callData);
void OnGenomeAT(Widget w, XtPointer data, XtPointer callData);
void OnGenomeGCSkew(Widget w, XtPointer data, XtPointer callData);
void OnGenomeATSkew(Widget w, XtPointer data, XtPointer callData);
void OnGenomeKetoSkew(Widget w, XtPointer data, XtPointer callData);
void OnGenomePurineSkew(Widget w, XtPointer data, XtPointer callData);
void OnGenomeTm(Widget w, XtPointer data, XtPointer callData);
void OnGenomeUser(Widget w, XtPointer data, XtPointer callData);
void OnGenomeConservation(Widget w, XtPointer data, XtPointer callData);

void OnGenomeMoveUp(Widget w, XtPointer data, XtPointer callData);
void OnGenomeMoveDown(Widget w, XtPointer data, XtPointer callData);
void OnGenomeShowMore(Widget w, XtPointer data, XtPointer callData);
void OnGenomeShowLess(Widget w, XtPointer data, XtPointer callData);
void OnGenomeInvert(Widget w, XtPointer data, XtPointer callData);

void OnSequenceDNA(Widget w, XtPointer data, XtPointer callData);
void OnSequenceProtein(Widget w, XtPointer data, XtPointer callData);

void OnColorBackground(Widget w, XtPointer data, XtPointer callData);
void OnColorBackgroundOk(Widget w, XtPointer data, XtPointer callData);

void OnColorSelection(Widget w, XtPointer data, XtPointer callData);
void OnColorSelectionOk(Widget w, XtPointer data, XtPointer callData);

void OnColorText(Widget w, XtPointer data, XtPointer callData);
void OnColorTextOk(Widget w, XtPointer data, XtPointer callData);

void OnColorGene(Widget w, XtPointer data, XtPointer callData);
void OnColorGeneOk(Widget w, XtPointer data, XtPointer callData);

void OnColorPseudoGene(Widget w, XtPointer data, XtPointer callData);
void OnColorPseudoGeneOk(Widget w, XtPointer data, XtPointer callData);

void OnColorCDS(Widget w, XtPointer data, XtPointer callData);
void OnColorCDSOk(Widget w, XtPointer data, XtPointer callData);

void OnColorAnnot(Widget w, XtPointer data, XtPointer callData);
void OnColorAnnotOk(Widget w, XtPointer data, XtPointer callData);

void OnColorRNA(Widget w, XtPointer data, XtPointer callData);
void OnColorRNAOk(Widget w, XtPointer data, XtPointer callData);

void OnColorIntergenic(Widget w, XtPointer data, XtPointer callData);
void OnColorIntergenicOk(Widget w, XtPointer data, XtPointer callData);

void OnColorCustom(Widget w, XtPointer data, XtPointer callData);
void OnColorCustomOk(Widget w, XtPointer data, XtPointer callData);

void HandleClose(Widget w, void (*CB)(Widget, XtPointer, XtPointer));

void ensurePulldownColormapInstalled(Widget w, XtPointer clientData, XtPointer callData);
void draw(Widget w, XtPointer data, XtPointer callData);
void postRedisplay();
XBool handleRedisplay(XtPointer closure);
void resize(Widget w, XtPointer data, XtPointer callData);
void init(Widget w, XtPointer data, XtPointer callData);
void swap(void);
void UpdateStatusBarInfo(const string &m_info);
void UpdateStatusBar();

void Button1Down(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button1Up(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button2Down(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button2Up(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button3Down(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button3Up(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);			  
void MouseMove(Widget w, XEvent * event, String * params,
	Cardinal * num_params);	

// Note that buttons 4 and 5 represent the mouse wheel	
void Button4Down(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
void Button5Down(Widget w, XEvent * event, String * params, 
	Cardinal * num_params);
		
void OnKeyDown(Widget w, XEvent * event, String * params,
	Cardinal * num_params);	

Widget GetTopShell(Widget w);	
void AfxMessageBox(const string &m_error);
void ColorDialog(Widget w, XtPointer data, XtPointer callData);
void ColorDialogClose(Widget w, XtPointer data, XtPointer callData);
void ColorDialogR(Widget w, XtPointer data, XtPointer callData);
void ColorDialogG(Widget w, XtPointer data, XtPointer callData);
void ColorDialogB(Widget w, XtPointer data, XtPointer callData);

void InfoDialog(Widget w, XtPointer data, XtPointer callData);
void InfoDialogClose(Widget w, XtPointer data, XtPointer callData);

void FindDialog(Widget w, XtPointer data, XtPointer callData);
void FindDialogSearchKey(Widget w, XtPointer data, XtPointer callData);
void FindDialogSearch(Widget w, XtPointer data, XtPointer callData);
void FindDialogSearchAgain(Widget w, XtPointer data, XtPointer callData);
void FindDialogAnnotate(Widget w, XtPointer data, XtPointer callData);
void FindDialogClose(Widget w, XtPointer data, XtPointer callData);

void PrimerDialog(Widget w, XtPointer data, XtPointer callData);
void PrimerDialogOk(Widget w, XtPointer data, XtPointer callData);
void PrimerDialogClose(Widget w, XtPointer data, XtPointer callData);

void ExportGenomeDialog(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeDialogOk(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeDialogClose(Widget w, XtPointer data, XtPointer callData);
void ExportGenome(Widget w, XtPointer data, XtPointer callData);

void ExportGenomeToggleFastaGenome(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeToggleFastaDNA(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeToggleFastaProtein(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeToggleGBK(Widget w, XtPointer data, XtPointer callData);
void ExportGenomeTogglePTT(Widget w, XtPointer data, XtPointer callData);

void DownloadDialog(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogKey(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogSearch(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogLoad(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogLoadAndSave(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogListCallback(Widget w, XtPointer data, XtPointer callData);
void DownloadDialogClose(Widget w, XtPointer data, XtPointer callData);

void HybridDialog(Widget w, XtPointer data, XtPointer callData);
void HybridDialogSearchPrimers(Widget w, XtPointer data, XtPointer callData);
void HybridDialogSearchProbe(Widget w, XtPointer data, XtPointer callData);
void HybridDialogShowNext(Widget w, XtPointer data, XtPointer callData);
void HybridDialogAnnotate(Widget w, XtPointer data, XtPointer callData);
void HybridDialogClose(Widget w, XtPointer data, XtPointer callData);

void PadlockDialog(Widget w, XtPointer data, XtPointer callData);
void PadlockDialogSearchPrimers(Widget w, XtPointer data, XtPointer callData);
void PadlockDialogSearchProbe(Widget w, XtPointer data, XtPointer callData);
void PadlockDialogShowNext(Widget w, XtPointer data, XtPointer callData);
void PadlockDialogAnnotate(Widget w, XtPointer data, XtPointer callData);
void PadlockDialogClose(Widget w, XtPointer data, XtPointer callData);

void OptionsDialog(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogFontScale(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogGCWindow(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogTmWindow(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogSalt(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogStrand(Widget w, XtPointer data, XtPointer callData);
void OptionsDialogClose(Widget w, XtPointer data, XtPointer callData);

void PasteClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params);
	
void CopyClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params);	

void CutClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params);

void OnHScroll(Widget w, XtPointer data, XtPointer callData);
void OnHScrollSnapBack(Widget w, XtPointer data, XtPointer callData);
void OnHScrollAndSnap(Widget w, XtPointer data, XtPointer callData);

void OnVScroll(Widget w, XtPointer data, XtPointer callData);
void OnVScrollSnapBack(Widget w, XtPointer data, XtPointer callData);
void OnVScrollAndSnap(Widget w, XtPointer data, XtPointer callData);

static XtActionsRec actionsTable[] = {
	{"Button1Down", Button1Down},
	{"Button1Up", 	 Button1Up},
	{"Button1Move", MouseMove},
	{"Button2Down", Button2Down},
	{"Button2Up", 	 Button2Up},
	{"Button2Move", MouseMove},
	{"Button3Down", Button3Down},
	{"Button3Up", 	 Button3Up},
	{"Button3Move", MouseMove},
	{"MouseMove", MouseMove},
	{"OnKeyDown", OnKeyDown},
	{"Button4Down", Button4Down},
	{"Button5Down", Button5Down},
	{"pasteclipboard", PasteClipboard},
    	{"copyclipboard", CopyClipboard},
    	{"cutclipboard", CutClipboard}
};

static char *glxareaTranslations =
  "#override\n\
  <Btn1Down>:Button1Down()\n\
  <Btn1Up>:Button1Up()\n\
  <Btn2Down>:Button2Down()\n\
  <Btn2Up>:Button2Up()\n\
  <Btn3Down>:Button3Down()\n\
  <Btn3Up>:Button3Up()\n\
  <Btn4Down>:Button4Down()\n\
  <Btn5Down>:Button5Down()\n\
  <Motion>:MouseMove()\n\
  <Key>:OnKeyDown()";
  
// Mouse controls
void Button1Down(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button1Down: NULL ptrGP";
	}
	
	bool update_display = false;
	bool double_click = false;
	
	// Thanks to Motifs mouse handling, we need to fake a double click
	static struct timeval last_click;
	static bool first_time = true;
	
	if(first_time){
		gettimeofday(&last_click, NULL);
		
		first_time = false;
	}
	else{
		struct timeval cur_click;
		
		// Fine tune this!
		const int double_click_threshold = 500000; // 0.5 sec
		
		gettimeofday(&cur_click, NULL);
		
		if((cur_click.tv_sec == last_click.tv_sec) && 
		   (cur_click.tv_usec - last_click.tv_usec < double_click_threshold)){
			double_click = true;
		}
		
		last_click = cur_click;
	}
	
	if(double_click){
		string info;
		
		if(ptrGP->l_button_dbl_down(event->xbutton.x, event->xbutton.y, info) == true){

			if(info == ""){
				info = "No information to display";
			}
			
			InfoDialog(XtParent(w), (char*)info.c_str(), NULL);
			
			update_display = true;
		}
	}
	else{
		
		string info;
		
		switch(event->xkey.state){
			case ControlMask:
				update_display = 
					ptrGP->l_button_ctrl_down(event->xbutton.x, event->xbutton.y);
					
				break;
			case ShiftMask:
				update_display = 
					ptrGP->l_button_shift_down(event->xbutton.x, event->xbutton.y, info);

					if(!info.empty()){
						InfoDialog(XtParent(w), (char*)info.c_str(), NULL);
					}

				break;
			default:
				update_display = ptrGP->l_button_down(event->xbutton.x, event->xbutton.y);
				
				break;
		};
	}
	
	if(update_display){
		postRedisplay();
	}
}

void Button1Up(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button1Up: NULL ptrGP";
	}
	
	if( ptrGP->l_button_up(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
}

void Button2Down(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button2Down: NULL ptrGP";
	}
	
	if(ptrGP->m_button_down(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
}

void Button2Up(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button2Down: NULL ptrGP";
	}
	
	if(ptrGP->m_button_up(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
}

void Button3Down(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button3Down: NULL ptrGP";
	}
	
	if(ptrGP->r_button_down(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
}

void Button3Up(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button3Down: NULL ptrGP";
	}
	
	if(ptrGP->r_button_up(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
}

void Button4Down(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button4Down: NULL ptrGP";
	}
	
	const int zDelta = 10;
	
	// Button 4 represents the mouse wheel moving up
	if(ptrGP->mouse_wheel(event->xbutton.x, event->xbutton.y, zDelta) == true){

		postRedisplay();
	}
	
	UpdateStatusBar();
}

void Button5Down(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":Button5Down: NULL ptrGP";
	}
	
	const int zDelta = -10;
	
	// Button 5 represents the mouse wheel moving down
	if(ptrGP->mouse_wheel(event->xbutton.x, event->xbutton.y, zDelta) == true){

		postRedisplay();
	}
	
	UpdateStatusBar();
}

void MouseMove(Widget w, XEvent * event, String * params, Cardinal * num_params)
{
	if(NULL == ptrGP){
		throw __FILE__ ":MouseMove: NULL ptrGP";
	}
	
	if(ptrGP->mouse_move(event->xbutton.x, event->xbutton.y) == true){
		postRedisplay();
	}
	
	// Always update the current base position
	UpdateStatusBar();
}

void OnKeyDown(Widget w, XEvent * event, String * params,
	Cardinal * num_params)
{
	if(ptrGP == NULL){
		return;
	}
	
	bool update_view = false;
	
	KeySym key = XKeycodeToKeysym(dpy, event->xkey.keycode, 0);

	// Manage keyboard events
	switch(key){
		case XK_Left:
		case XK_KP_Left:
			update_view = ptrGP->key_left_arrow();
			break;
		case XK_Right:
		case XK_KP_Right:
			update_view = ptrGP->key_right_arrow();
			break;
		case XK_Up:
		case XK_KP_Up:
			
			if(event->xkey.state == ShiftMask){
				update_view = ptrGP->shift_selection_up();
			}
			else{
				update_view = ptrGP->key_up_arrow();
			}
			break;
		case XK_Down:
		case XK_KP_Down:
			if(event->xkey.state == ShiftMask){
				update_view = ptrGP->shift_selection_down();
			}
			else{
				update_view = ptrGP->key_down_arrow();
			}
			break;
		case XK_Page_Up:
		case XK_KP_Page_Up:
			update_view = ptrGP->key_page_up();
			break;
		case XK_Page_Down:
		case XK_KP_Page_Down:
			update_view = ptrGP->key_page_down();
			break;
		// Disable the keyboard shortcuts for deleting genomes
		// and annotation. It's too easy to accidently delete items
		// that we can't restore (until undo functionality is added).
		//case XK_Delete:
		//case XK_KP_Delete:
		//	if(event->xkey.state == ControlMask){
		//		OnGenomeAnnotationDelete(NULL, NULL, NULL);
		//	}
		//	else{
		//		update_view = ptrGP->delete_selection();
		//	}
		//	break;
		case XK_Escape:
			update_view = ptrGP->clear_selection();
			ptrGP->jump_buffer_clear();
			break;
		case XK_F3:
			FindDialogSearchAgain(XtParent(w), NULL, NULL);
			break;
		case 'g':
			if(event->xkey.state == ShiftMask){
				ptrGP->jump_to();
				ptrGP->jump_buffer_clear();
				update_view = true;
			}
						
			break;
		case '=': // Shift + '=' = '+'
			// If this is not the '+' key, then break. Otherwise
			// fall through to the '+' function
			if(event->xkey.state != ShiftMask){
				break;
			}
		case XK_KP_Add:
			
			update_view = ptrGP->key_plus();
			break;
		case '-':
		case XK_KP_Subtract:
			
			update_view = ptrGP->key_minus();
			break;
	};
	
	// Note that isdigit does not like arguments of 
	// type KeySym (i.e. seg fault if ctrl key is pressed)
	if( (key >= '0') && (key <= '9') ){
		ptrGP->jump_buffer_push(key - '0');
	}
	
	if(update_view){
		postRedisplay();
	}
}

static void detectOverlaySupport(Display *dpy)
{	
	sovVisualInfo sovtemplate, *overlayVisuals;
	int layer, nVisuals, i, entries;

	/* Need more than two colormap entries for reasonable menus. */
	entries = 2;
	for (layer = 1; layer <= 3; layer++) {
		sovtemplate.layer = layer;
    		sovtemplate.vinfo.screen = DefaultScreen(dpy);
    		overlayVisuals = sovGetVisualInfo(dpy,
      		VisualScreenMask | VisualLayerMask, &sovtemplate, &nVisuals);
    		if(overlayVisuals) {
      		for(i = 0; i < nVisuals; i++) {
        			if(overlayVisuals[i].vinfo.visual->map_entries > entries){
          			overlayVisual = overlayVisuals[i].vinfo.visual;
          			overlayDepth = overlayVisuals[i].vinfo.depth;
	  					entries = overlayVisual->map_entries;
        			}
      		}
      		XFree(overlayVisuals);
    		}
  	}
	
  	if(overlayVisual){
		overlayColormap = XCreateColormap(dpy, DefaultRootWindow(dpy),
      					overlayVisual, AllocNone);
	}
}

void quit(Widget w, XtPointer data, XtPointer callData)
{	
	XtAppSetExitFlag(app);
}

// Map the "close" window manager button to an appropriate
// function (i.e. Cancel()).
void HandleClose(Widget w, void (*CB)(Widget, XtPointer, XtPointer))
{
	Atom WM_DELETE_WINDOW;
     WM_DELETE_WINDOW = XmInternAtom(XtDisplay(w), "WM_DELETE_WINDOW", 0);
	
   	XmAddProtocolCallback(w,
		  XM_WM_PROTOCOL_ATOM(w),
        	  WM_DELETE_WINDOW,
        	  CB,
         	  (XtPointer) w);
}

void ensurePulldownColormapInstalled(Widget w, XtPointer clientData, XtPointer callData)
{
  /* Ensure that overlay pulldown menu's colormap is installed. */
  XInstallColormap(dpy, overlayColormap);
}

void draw(Widget w, XtPointer data, XtPointer callData)
{
  	postRedisplay();
}

void postRedisplay(void)
{
  	if(!redisplayPending) {
    		XtAppAddWorkProc(app, handleRedisplay, 0);
    		redisplayPending = true;
  	}
}

XBool handleRedisplay(XtPointer closure)
{
  	if(ptrGP){
		if(ptrGP->render() == true){
			// Don't use postRedisplay() because the redisplayPending flag
			// allows at most one request in the event queue.
			XtAppAddWorkProc(app, handleRedisplay, 0);
		}
		
		swap();
	
		redisplayPending = false;
	}
			
  	return True;
}

void resize(Widget w, XtPointer data, XtPointer callData)
{
	GLwDrawingAreaCallbackStruct *resize = (GLwDrawingAreaCallbackStruct*) callData;

	if (madeCurrent) {
    		glXMakeCurrent(dpy, glxwin, cx);
    		glXWaitX();
    
    		viewWidth = resize->width;
		viewHeight = (resize->height == 0) ? 0 : resize->height - HEIGHT_OFFSET;		
  	}
	
	if(ptrGP){
		ptrGP->setup_view_OGL(viewWidth, viewHeight);
	}
}

void init(Widget w, XtPointer data, XtPointer callData)
{
	glxwin = XtWindow(w);
  	glXMakeCurrent(XtDisplay(w), XtWindow(w), cx);	
}

void swap(void)
{
	glXSwapBuffers(dpy, glxwin);
}

void UpdateStatusBarInfo(const string &m_info)
{
	XmString str = XmStringCreateLocalized((char*)(m_info.c_str()));
	
	XtVaSetValues(status_bar_info, XmNlabelString, str, NULL);
	
	XmStringFree(str);
}

void UpdateStatusBar()
{
	if(ptrGP == NULL){
		return;
	}
	
	std::stringstream base;
	
	base << "Base(" << ptrGP->cur_base_loc() << ")";
	
	base.precision(3);
	
	// Is the user displaying Tm in the current genome?
	if(ptrGP->cur_show_tm()){
		base << ":Tm(" << ptrGP->cur_tm() << ")";
	}
	
	// Is the user displaying %G+C in the current genome?
	if(ptrGP->cur_show_gc()){
		base << ":%G+C(" << ptrGP->cur_gc() << ")";
	}
	
	// Is the user displaying %A+T in the current genome?
	if(ptrGP->cur_show_at()){
		base << ":%A+T(" << ptrGP->cur_at() << ")";
	}
	
	// Is the user displaying GC skew in the current genome?
	if(ptrGP->cur_show_gc_skew()){
		base << ":GC skew(" << ptrGP->cur_gc_skew() << ")";
	}
	
	// Is the user displaying AT skew in the current genome?
	if(ptrGP->cur_show_at_skew()){
		base << ":AT skew(" << ptrGP->cur_at_skew() << ")";
	}
	
	// Is the user displaying Keto skew in the current genome?
	if(ptrGP->cur_show_keto_skew()){
		base << ":Keto(" << ptrGP->cur_keto_skew() << ")";
	}
	
	// Is the user displaying Purine skew in the current genome?
	if(ptrGP->cur_show_purine_skew()){
		base << ":Purine(" << ptrGP->cur_purine_skew() << ")";
	}
	
	// Is the user displaying Genome plot in the current genome?
	if(ptrGP->cur_show_genome_plot()){
		base << ":user(" << ptrGP->cur_genome_plot() << ")";
	}
	
	base << "|";
	
	XmString str = XmStringCreateLocalized( (char*)(base.str().c_str()) );
	
	XtVaSetValues(status_bar_base, XmNlabelString, str, NULL);
	
	XmStringFree(str);
}

int main(int argc, char *argv[])
{	
	toplevel = XtAppInitialize(&app, "Genomorama", NULL, 0, &argc, argv,
		fallbackResources, NULL, 0);
	
	//toplevel = XtOpenApplication(&app, "Genomorama", NULL, 0, &argc, argv,
	//	fallbackResources, applicationShellWidgetClass, NULL, 0);		
			
	// Make sure we play nice when the user closes the main
	// window with the window manager (i.e. make sure we 
	// deallocate all of the memory correctly).
	HandleClose(toplevel, quit);

	XtAppAddActions(app, actionsTable, XtNumber(actionsTable));
	
	dpy = XtDisplay(toplevel);
	
	// vi is a global variable, make sure that it is
	// initialized properly
	vi = NULL;
	
	for(int depth = 32;(depth >= 16) && (vi == NULL);depth -= 4){ 
		for(int num_sample = 16;num_sample > 1;num_sample--){

			int config[] = {
				  GLX_DOUBLEBUFFER, GLX_RGBA, GLX_DEPTH_SIZE, depth,
				  GLX_SAMPLE_BUFFERS_ARB, 1, GLX_SAMPLES_ARB, num_sample, None
				};

			// Find an OpenGL-capable RGB visual with depth buffer.
			vi = glXChooseVisual(dpy, DefaultScreen(dpy), config);

			if(vi != NULL){
				cerr << "Using " << num_sample << "x multi-sampling" << endl;
				cerr << "Using " << depth << " bit depth" << endl;
				break;
			}
		}
	}
	
	if (vi == NULL) {
		
		int config[] = {
			  GLX_DOUBLEBUFFER, GLX_RGBA, GLX_DEPTH_SIZE, 16,
			  None
			};
			
		vi = glXChooseVisual(dpy, DefaultScreen(dpy), config);
		
		if(vi != NULL){
			cerr << "Using vanilla double buffered display" << endl;
		}
	}
	
	if (vi == NULL) {
		
		int config[] = {
			  GLX_RGBA, GLX_DEPTH_SIZE, 12,
			  None
			};
			
		vi = glXChooseVisual(dpy, DefaultScreen(dpy), config);
		
      	if (vi == NULL){
			XtAppError(app, "no RGB visual with depth buffer");
		}
		else{
			cerr << "Using vanilla single buffered display" << endl;
		}
		
		doubleBuffer = 0;
	}

	// Create an OpenGL rendering context
	// Note that direct rendering (enabled with the last argument set to
	// true) leads to visual artifacts around the mouse (as well as the
	// mouse autohiding). This may just be a bug with my setup, but I'll
	// leave direct rendering disabled for now.
	#ifdef DISABLE_DRI
	cx = glXCreateContext(dpy, 
		vi, 
		None, 	// No display list sharing
		false); 	// Favor direct rendering?
	#else
	cx = glXCreateContext(dpy, 
		vi, 
		None, 	// No display list sharing
		true); 	// Favor direct rendering?
	#endif // DISABLE_DRI
								 
	if(cx == NULL){
		XtAppError(app, "could not create rendering context");
	}
		
	mainw = XtVaCreateWidget("mainw", xmMainWindowWidgetClass, 
				toplevel,
				XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE, 
				NULL);
	
	// Add the application icon
	icon = XCreateBitmapFromData(dpy,
		 RootWindowOfScreen(XtScreen(toplevel)), (char *)genomorama_bits,
		 genomorama_width, genomorama_height);
		 
	icon_mask = XCreateBitmapFromData(dpy,
		RootWindowOfScreen(XtScreen(toplevel)), (char *)genomorama_mask_bits,
		genomorama_width, genomorama_height);
	
	XtVaSetValues(toplevel,
		XmNiconPixmap, icon, 
		XmNiconMask, icon_mask, NULL);
	
	XtManageChild(mainw);
		
  	// Try to find a good overlay visual for pulldown and popup menus.
  	detectOverlaySupport(dpy);
	
	/////////////////////////////////////////////////////////////////////
	// Create the menu bar.
	
	int n = 0;
	
	if (overlayVisual) {
		XtSetArg(args[n], XmNvisual, overlayVisual);
    		n++;
    		XtSetArg(args[n], XmNdepth, overlayDepth);
    		n++;
    		XtSetArg(args[n], XmNcolormap, overlayColormap);
    		n++;
  	}
  	
	// A temporary argument for setting a single widget property
  	Arg tmp_arg[1];
	
	// Turn of the menu bar shadow -- the 80's are over!
  	XtSetArg(tmp_arg[0], XmNshadowThickness, 0);
	//XtSetArg(tmp_arg[1], XmNkeyboardFocusPolicy, XmEXPLICIT);
	
	menubar = XmCreateMenuBar(mainw, "menubar", tmp_arg, 2);
	XtManageChild(menubar);
		
  	// Create the "File" menu.
  	menupane = XmCreatePulldownMenu(menubar, "file", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
  
	AddMenuItem(menupane, "open", "Open ...", 'O', OnFileOpen);
	
	AddMenuItem(menupane, "plot", "Load Genome Plot ...", 'L', 
		OnFileLoadGenomePlot);
	
	AddMenuItem(menupane, "download", "Download ...", 'D', 
		DownloadDialog);
	
	AddMenuSeperator(menupane);
	
	// Create the "Export" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "export", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "postscript", "Postscript ...", 'P', 
			OnFileExportPs);
		
		AddMenuItem(submenupane, "gif", "GIF ...", 'I', 
			OnFileExportGif);
		
		AddMenuItem(submenupane, "genomes", "Genomes ...", 'G', 
			OnFileExportGenomes);
			
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Export", tmp_arg, 1);
  		XtManageChild(cascade);
	
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "exit", "Exit", 'x', quit);
			
  	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "File", tmp_arg, 1);
  	XtManageChild(cascade);
	
	// Create the "Edit" menu.
  	menupane = XmCreatePulldownMenu(menubar, "edit", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
  
  	AddMenuItem(menupane, "delete_genome", "Delete Genome", 'D', 
		OnEditDelete);
		
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "select_all", "Select All", 'S', 
		OnEditSelectAll);
		  	
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "find", "Find ...", 'F', 
		OnEditFind);
	
	AddMenuItem(menupane, "find_again", "Find Again...", 'A', 
		OnEditFindAgain);
			
	AddMenuSeperator(menupane);
	
	// Create the "Hybridize" submenu.
	
	submenupane = XmCreatePulldownMenu(menupane, "submenupane", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "hybridize_pcr", "PCR/Probe ...", 'P', 
			OnEditHybridize);
		
		AddMenuItem(submenupane, "hybridize_padlock", "Padlock Probe ...", 'l', 
			OnEditHybridizePadlock);
							
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Hybridize", tmp_arg, 1);
  		XtManageChild(cascade);
	
	// Create the "Primers" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "submenupane", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "pick_primers", "Pick Primers ...", 'P', 
			OnEditPrimer);
		
		AddMenuItem(submenupane, "delete_primers", "Delete Primers ...", 'D', 
			OnEditPrimerDelete);
				
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Primers", tmp_arg, 1);
  		XtManageChild(cascade);
		
  	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "Edit", tmp_arg, 1);
  	XtManageChild(cascade);
	
	// Create the "View" menu.
  	menupane = XmCreatePulldownMenu(menubar, "view", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
  	
  	size_centric_toggle = XtVaCreateManagedWidget("Size centric",
        	xmToggleButtonWidgetClass, menupane, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK,
    		NULL);
	
  	XtAddCallback(size_centric_toggle, XmNvalueChangedCallback, 
		OnViewSizeCentric, NULL);
	
	// The default view is size centric
	XmToggleButtonSetState(size_centric_toggle, 
		True, False);
	
  	XtManageChild(size_centric_toggle);
	
  	gene_centric_toggle = XtVaCreateManagedWidget("Gene centric",
        	xmToggleButtonWidgetClass, menupane, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK,
    		NULL);
	
  	XtAddCallback(gene_centric_toggle, XmNvalueChangedCallback, 
		OnViewGeneCentric, NULL);
		
	// The default view is *not* gene centric
	XmToggleButtonSetState(gene_centric_toggle, 
		False, False);
		
  	XtManageChild(gene_centric_toggle);
	
	AddMenuSeperator(menupane);
	
	btn = XtVaCreateManagedWidget("Magnify",
        	xmToggleButtonWidgetClass, menupane, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK,
    		NULL);
	
  	XtAddCallback(btn, XmNvalueChangedCallback, 
		OnViewMagnify, NULL);
		
	// By default, Magnify is off
	XmToggleButtonSetState(btn, 
		False, False);
		
  	XtManageChild(btn);
	
	AddMenuItem(menupane, "fit_to_window", "Fit to window", 'F',
		OnViewFitToWindow);
		
  	btn = XtVaCreateManagedWidget("Momentum",
        	xmToggleButtonWidgetClass, menupane, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK,
    		NULL);
	
  	XtAddCallback(btn, XmNvalueChangedCallback, 
		OnViewMomentum, NULL);
		
	// By default, momentum is off
	XmToggleButtonSetState(btn, 
		False, False);
		
  	XtManageChild(btn);
		
	AddMenuItem(menupane, "options", "Options ...", 'O', 
			OnViewOptions);
					
  	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "View", tmp_arg, 1);
  	XtManageChild(cascade);
	
	// Create the "Genome" menu.
  	menupane = XmCreatePulldownMenu(menubar, "genome", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
  	
	AddMenuItem(menupane, "edit_annotation", "Edit Annotation ...", 'E',
		OnGenomeAnnotationEdit);
		
	AddMenuItem(menupane, "import_annotation", "Import Annotation ...", 'I', 
		OnGenomeAnnotationImport);
		
  	AddMenuItem(menupane, "delete_annotation", "Delete Annotation", 'D', 
		OnGenomeAnnotationDelete);
				
  	// Create the "Hide" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "hide", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "genome", "Genome", 'G', 
			OnGenomeHide);
		
		AddMenuItem(submenupane, "annotation", "Annotation", 'A', 
			OnGenomeAnnotationHide);
			
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Hide", tmp_arg, 1);
  		XtManageChild(cascade);
	
	// Create the "Unhide" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "unhide", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "unhide_all_genomes", "Unhide All Genomes", 'G', 
			OnGenomeShowAll);
			
		AddMenuItem(submenupane, "unhide_annotation", "Unhide Annotation", 'A', 
			OnGenomeShowAnnotation);
		
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Unhide", tmp_arg, 1);
  		XtManageChild(cascade);
	
	AddMenuSeperator(menupane);
	
	// Create the "Plot" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "plot", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "g+c", "%G+C", 'G', 
			OnGenomeGC);
			
		AddMenuItem(submenupane, "a+t", "%A+T", 'A', 
			OnGenomeAT);
		
		AddMenuItem(submenupane, "gc_skew", "GC Skew", 'C', 
			OnGenomeGCSkew);
		
		AddMenuItem(submenupane, "at_skew", "AT Skew", 'T', 
			OnGenomeATSkew);
		
		AddMenuItem(submenupane, "keto_skew", "Keto Skew", 'K', 
			OnGenomeKetoSkew);
		
		AddMenuItem(submenupane, "purine_skew", "Purine Skew", 'P', 
			OnGenomePurineSkew);
		
		AddMenuItem(submenupane, "tm", "Tm", 'm', 
			OnGenomeTm);
		
		AddMenuItem(submenupane, "user", "User", 'U', 
			OnGenomeUser);
		
		// Create the "Alignment" submenu.
		subsubmenupane = XmCreatePulldownMenu(submenupane, "alignment", args, n);

			if (overlayVisual) {
    				XtAddCallback(XtParent(subsubmenupane), XmNpopupCallback,
      					    ensurePulldownColormapInstalled, NULL);
  			}
			
			AddMenuItem(subsubmenupane, "conservation", "Conservation", 'C', 
				OnGenomeConservation);
			
			XtSetArg(tmp_arg[0], XmNsubMenuId, subsubmenupane);
  			cascade = XmCreateCascadeButton(submenupane, "Alignment", tmp_arg, 1);
  			XtManageChild(cascade);
		
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Plot", tmp_arg, 1);
  		XtManageChild(cascade);
		
	// Create the "Move" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "move", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "up", "Up", 'U', 
			OnGenomeMoveUp);
		
		AddMenuItem(submenupane, "down", "Down", 'D', 
			OnGenomeMoveDown);
	
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Move", tmp_arg, 1);
  		XtManageChild(cascade);
	
	// Create the "Show" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "show", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "one_more", "One More", 'M', 
			OnGenomeShowMore);
		
		AddMenuItem(submenupane, "one_less", "One Less", 'L', 
			OnGenomeShowLess);
	
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "Show", tmp_arg, 1);
  		XtManageChild(cascade);
	
	AddMenuItem(menupane, "complement", "Complement", 'C', 
			OnGenomeInvert);
	
  	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "Genome", tmp_arg, 1);
  	XtManageChild(cascade);
	
	// Create the "Sequence" menu.
  	menupane = XmCreatePulldownMenu(menubar, "menupane", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
	
	// Create the "FASTA" submenu.
	submenupane = XmCreatePulldownMenu(menupane, "submenupane", args, n);
	
		if (overlayVisual) {
    			XtAddCallback(XtParent(submenupane), XmNpopupCallback,
      				    ensurePulldownColormapInstalled, NULL);
  		}
		
		AddMenuItem(submenupane, "dna", "DNA", 'D', 
			OnSequenceDNA);
					
		AddMenuItem(submenupane, "protein", "Protein", 'P', 
			OnSequenceProtein);
	
		XtSetArg(tmp_arg[0], XmNsubMenuId, submenupane);
  		cascade = XmCreateCascadeButton(menupane, "FASTA", tmp_arg, 1);
  		XtManageChild(cascade);
		
	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "Sequence", tmp_arg, 1);
  	XtManageChild(cascade);
	
	// Create the "Color" menu.
  	menupane = XmCreatePulldownMenu(menubar, "menupane", args, n);
	
  	if (overlayVisual) {
    		XtAddCallback(XtParent(menupane), XmNpopupCallback,
      			    ensurePulldownColormapInstalled, NULL);
  	}
  
	AddMenuItem(menupane, "background", "Background ...", 'B', 
			OnColorBackground);
				
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "selection", "Selection ...", 'S', 
			OnColorSelection);
	
	AddMenuItem(menupane, "text", "Text ...", 'T', 
			OnColorText);
				
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "gene", "Gene ...", 'G', 
			OnColorGene);
	
	AddMenuItem(menupane, "pseudo_gene", "Pseudo-gene ...", 'P', 
			OnColorPseudoGene);
	
	AddMenuItem(menupane, "cds", "CDS(orf) ...", 'C', 
			OnColorCDS);
			
	AddMenuItem(menupane, "misc_annotation", "Misc Annotation ...", 'M', 
			OnColorAnnot);
	
	AddMenuItem(menupane, "rna", "RNA ...", 'R', 
			OnColorRNA);
	
	AddMenuItem(menupane, "intergeneic", "Intergeneic ...", 'I', 
			OnColorIntergenic);
				
	AddMenuSeperator(menupane);
	
	AddMenuItem(menupane, "custom", "Custom ...", 'u', 
			OnColorCustom);
				
  	XtSetArg(tmp_arg[0], XmNsubMenuId, menupane);
  	cascade = XmCreateCascadeButton(menubar, "Color", tmp_arg, 1);
  	XtManageChild(cascade);
		
  	/////////////////////////////////////////////////////////////////////
	// Create the status bar.
	status_bar = XtVaCreateWidget("status_bar",
		xmRowColumnWidgetClass,
		mainw,
		XmNisAligned, True,
		XmNentryAlignment, XmALIGNMENT_END,
		XmNorientation, XmHORIZONTAL,
		NULL);
	
	status_bar_base = XtVaCreateManagedWidget("Base: 0 | ", xmLabelWidgetClass, 
		status_bar,
		NULL);
	
	status_bar_info = XtVaCreateManagedWidget("Ready", xmLabelWidgetClass, 
		status_bar,
		NULL);
		
	XtManageChild(status_bar);
	
	/////////////////////////////////////////////////////////////////////
	// Create the scroll bars
	horizontal_scroll_bar =  XtVaCreateManagedWidget("horizontal", xmScrollBarWidgetClass , mainw,
		XmNsliderSize, 2,
		XmNvalue, HSCROLL_CENTER,
		XmNmaximum, HSCROLL_MAX,
		XmNminimum, HSCROLL_MIN,
		XmNincrement, HSCROLL_DELTA,
		XmNorientation, XmHORIZONTAL,
		NULL);
	
	XtAddCallback(horizontal_scroll_bar, XmNvalueChangedCallback, 
		OnHScrollSnapBack, NULL);
	
	XtAddCallback(horizontal_scroll_bar, XmNdragCallback, 
		OnHScroll, NULL);
	
	XtAddCallback(horizontal_scroll_bar, XmNdecrementCallback, 
		OnHScrollAndSnap, NULL);
	
	XtAddCallback(horizontal_scroll_bar, XmNincrementCallback, 
		OnHScrollAndSnap, NULL);
		
	XtAddCallback(horizontal_scroll_bar, XmNpageIncrementCallback,
		OnHScrollAndSnap, NULL);
	
	XtAddCallback(horizontal_scroll_bar, XmNpageDecrementCallback,
		OnHScrollAndSnap, NULL);
		
	XtManageChild(horizontal_scroll_bar);
	
	vertical_scroll_bar =  XtVaCreateManagedWidget("vertical", xmScrollBarWidgetClass , mainw,
		XmNsliderSize, 2,
		XmNvalue, VSCROLL_CENTER,
		XmNmaximum, VSCROLL_MAX,
		XmNminimum, VSCROLL_MIN,
		XmNincrement, VSCROLL_DELTA,
		XmNorientation, XmVERTICAL,
		NULL);
	
	XtAddCallback(vertical_scroll_bar, XmNvalueChangedCallback, 
		OnVScrollSnapBack, NULL);
	
	XtAddCallback(vertical_scroll_bar, XmNdragCallback, 
		OnVScroll, NULL);
	
	XtAddCallback(vertical_scroll_bar, XmNdecrementCallback, 
		OnVScrollAndSnap, NULL);
	
	XtAddCallback(vertical_scroll_bar, XmNincrementCallback, 
		OnVScrollAndSnap, NULL);
		
	XtAddCallback(vertical_scroll_bar, XmNpageIncrementCallback,
		OnVScrollAndSnap, NULL);
	
	XtAddCallback(vertical_scroll_bar, XmNpageDecrementCallback,
		OnVScrollAndSnap, NULL);
		
	XtManageChild(vertical_scroll_bar);	
	/////////////////////////////////////////////////////////////////////
	
  	/* Create framed drawing area for OpenGL rendering. */
  	frame = XmCreateFrame(mainw, "frame", NULL, 0);
  	XtManageChild(frame);
  	glxarea = XtVaCreateManagedWidget("glxarea", glwMDrawingAreaWidgetClass,
    							    frame, GLwNvisualInfo, vi, NULL);
  	XtVaGetValues(glxarea, XtNwidth, &viewWidth, XtNheight, &viewHeight, NULL);
  	XtAddCallback(glxarea, XmNexposeCallback, draw, NULL);
  	XtAddCallback(glxarea, XmNresizeCallback, resize, NULL);
  	XtAddCallback(glxarea, GLwNginitCallback, init, NULL);
  
  	/* Set up application's window layout. */
  	XmMainWindowSetAreas(mainw, menubar, status_bar, NULL, NULL, frame);
  	XtRealizeWidget(toplevel);
	
  	/* Once widget is realized (ie, associated with a created X
      window), we can bind the OpenGL rendering context to the
      window.  */

  	glXMakeCurrent(dpy, XtWindow(glxarea), cx);
  	madeCurrent = 1;

	if(trans == NULL){
    		trans = XtParseTranslationTable(glxareaTranslations);
		
		// Allow all main window related widgets to accept user input
    		XtOverrideTranslations(glxarea, trans);
		XtOverrideTranslations(mainw, trans);
		XtOverrideTranslations(status_bar, trans);
  	}
	
	try{
		ptrGP = new GenomePage;

		if(!ptrGP){
			throw __FILE__ ":main: Unable to allocate memory for GenomePage";
		}
		
		ptrGP->setup_view_OGL(viewWidth, viewHeight - HEIGHT_OFFSET);
		ptrGP->initialize_OGL();
		
		// Load fonts
		// Note that X fonts are not used because there is no easy way to 
		// get a vector based font out of GLX!
	
		LoadFont(ptrGP->get_font_metric());
		
		// Process any command line arguments
		for(int i = 1;i < argc;i++){
		
			try{
				cerr << "Reading data from " << argv[i] << endl;
				
				WaitCursor(glxarea);
				
				if(ptrGP->load( argv[i], file_type(argv[i]) ) == false){
					
					cerr << "Did not read any sequence data for this genome" << endl;
					cerr << "Try downloading this file directly from NCBI" << endl;
				}
				
				ArrowCursor(glxarea);
			}
			catch(const char *error){
				ArrowCursor(glxarea);
				
				cerr << "Error loading data from " << argv[i] 
					<< ": " << error << endl;
			}
			catch(...){
				ArrowCursor(glxarea);
				
				cerr << "Unknown error while loading data from " << argv[i] << endl;
			}
		}
		
  		XtAppMainLoop(app);
	}
	catch(const char* error){
		XtAppError(app, error);
	}
	catch(...){
		XtAppError(app, "An unknown error has occurred!");
	}
	
	if(ptrGP){
		delete ptrGP;
	}
	
  	return EXIT_SUCCESS;
}

void OnFileExportPs(Widget w, XtPointer data, XtPointer callData)
{
	// First, we need to get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*.ps", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, OnFileExportPsOk, NULL);
		
		// No need for a separate OnFileExportCancel -- just need to close the
		// dialog box
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void OnFileExportPsOk(Widget w, XtPointer data, XtPointer callData)
{
	// Copy the text from the file selection dialog to our
	// text widget.
	char *tmp_str = NULL;
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
	
	// Load the file!
	// Open the File
	try{
		if(ptrGP == NULL){
			throw __FILE__ ":OnFileExportPsOk: NULL ptrGP";
		}
		
		ptrGP->render_ps(file_name);
	}
	catch(const char *error){

		// Was there an error opening the file?
		AfxMessageBox(error);

		return;
	}
	catch(...){

		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to export postscript file!");

		return;
	}
}

void OnFileExportGif(Widget w, XtPointer data, XtPointer callData)
{
	// First, we need to get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*.gif", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, OnFileExportGifOk, NULL);
		
		// No need for a separate OnFileExportCancel -- just need to close the
		// dialog box
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void OnFileExportGifOk(Widget w, XtPointer data, XtPointer callData)
{
	// Copy the text from the file selection dialog to our
	// text widget.
	char *tmp_str = NULL;
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
	
	// Load the file!
	// Open the File
	try{
		if(ptrGP == NULL){
			throw __FILE__ ":OnFileExportGifOk: NULL ptrGP";
		}
		
		ptrGP->render_gif(file_name);
	}
	catch(const char *error){

		// Was there an error opening the file?
		AfxMessageBox(error);

		return;
	}
	catch(...){

		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to export gif file!");

		return;
	}
}

void OnFileExportGenomes(Widget w, XtPointer data, XtPointer callData)
{
	ExportGenomeDialog(XtParent(w), NULL, NULL);
}

void OnFileOpen(Widget w, XtPointer data, XtPointer callData)
{
	// First, we need to get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(GetTopShell(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, OnFileOpenOk, NULL);
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void OnFileOpenOk(Widget w, XtPointer data, XtPointer callData)
{
	// Copy the text from the file selection dialog to our
	// text widget.
	char *tmp_str = NULL;
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	if(ptrGP == NULL){
		throw __FILE__ ":OnFileOpenOk: NULL ptrGP";
	}
	
	WaitCursor(file_dialog);
	
	try{
		
		if(ptrGP->load(file_name, file_type(file_name)) == false){
			
			AfxMessageBox("Did not read any sequence data for this genome "
				"Try downloading this file directly from NCBI");
		}
	}
	catch(const char *error){
		
		// Was there an error opening the file?
		AfxMessageBox(error);
	}
	catch(...){
		
		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to read input file!");
	}
	
	ArrowCursor(file_dialog);
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
}

void OnFileOpenCancel(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
}

void OnFileLoadGenomePlot(Widget w, XtPointer data, XtPointer callData)
{
	// First, we need to get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, OnFileLoadGenomePlotOk, NULL);
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileLoadGenomePlotCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void OnFileLoadGenomePlotOk(Widget w, XtPointer text, XtPointer callData)
{
	// Copy the text from the file selection dialog to our
	// text widget.
	char *tmp_str = NULL;
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	WaitCursor(file_dialog);
	
	try{
		if(ptrGP == NULL){
			throw __FILE__ ":OnFileLoadGenomePlotOk: NULL ptrGP";
		}
		
		ptrGP->read_genome_plot(file_name);
		
		ArrowCursor(file_dialog);
	}
	catch(const char *error){
		
		ArrowCursor(file_dialog);
		
		// Was there an error opening the file?
		AfxMessageBox(error);
	}
	catch(...){
		
		ArrowCursor(file_dialog);
		
		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to read genome plot file!");
	}
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
}

void OnFileLoadGenomePlotCancel(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
}


void OnEditDelete(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnEditDelete: NULL ptrGP";
	}
		
	if(ptrGP->delete_selection() == true){
		postRedisplay();
	}
}

void OnEditSelectAll(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnEditSelectAll: NULL ptrGP";
	}
		
	if(ptrGP->select_all() == true){
		postRedisplay();
	}
}

void OnEditFind(Widget w, XtPointer data, XtPointer callData)
{
	FindDialog(XtParent(w), NULL, NULL);
}

void OnEditHybridize(Widget w, XtPointer data, XtPointer callData)
{
	HybridDialog(XtParent(w), NULL, NULL);
}

void OnEditHybridizePadlock(Widget w, XtPointer data, XtPointer callData)
{
	PadlockDialog(XtParent(w), NULL, NULL);
}

void OnEditFindAgain(Widget w, XtPointer data, XtPointer callData)
{
	FindDialogSearchAgain(XtParent(w), NULL, NULL);
}

void OnEditPrimer(Widget w, XtPointer data, XtPointer callData)
{
	PrimerDialog(XtParent(w), NULL, NULL);
}

void OnEditPrimerDelete(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnEditPrimerDelete: NULL ptrGP";
	}
		
	if(ptrGP->clear_primers() == true){
		postRedisplay();
	}
}

void OnViewOptions(Widget w, XtPointer data, XtPointer callData)
{
	OptionsDialog(XtParent(w), NULL, NULL);
}

void OnViewMomentum(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnViewMomentum: NULL ptrGP";
	}
	
	ptrGP->spin_momentum( !ptrGP->spin_momentum() );
}

void OnViewMagnify(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnViewMagnify: NULL ptrGP";
	}
	
	ptrGP->magnify( !ptrGP->magnify() );
	
	postRedisplay();
}

void OnViewFitToWindow(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnViewFitToWindow: NULL ptrGP";
	}
	
	ptrGP->fit_to_window();
	
	postRedisplay();
}

void OnViewSizeCentric(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnViewSizeCentric: NULL ptrGP";
	}
	
	ptrGP->mode(DNAMol::SIZE_CENTRIC);
	
	// Update the "centric" menus
	// Update the "centric" menus
	if(size_centric_toggle){
		 XmToggleButtonSetState(size_centric_toggle, True, False);
	}
	
	if(gene_centric_toggle){
		XmToggleButtonSetState(gene_centric_toggle, False, False);
	}
	
	postRedisplay();
}

void OnViewGeneCentric(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnViewSizeCentric: NULL ptrGP";
	}
	
	ptrGP->mode(DNAMol::GENE_CENTRIC);
	
	// Update the "centric" menus
	if(size_centric_toggle){
		 XmToggleButtonSetState(size_centric_toggle, False, False);
	}
	
	if(gene_centric_toggle){
		XmToggleButtonSetState(gene_centric_toggle, True, False);
	}
	
	postRedisplay();
}

void OnGenomeAnnotationDelete(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeAnnotationDelete: NULL ptrGP";
	}
		
	if(ptrGP->delete_annotation() == true){
		postRedisplay();
	}
	else{
		AfxMessageBox("Please select at least one annotation to delete");
	}
}

void OnGenomeAnnotationImport(Widget w, XtPointer data, XtPointer callData)
{
	if( (ptrGP == NULL) || (ptrGP->num_display() == 0) ){
		AfxMessageBox("Please open a genome before importing additional annotation");
		return;
	}
	
	// First, we need to get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
		// Any file that contains annotation can be used (i.e. *.asn, *.gbk, *.ptt)
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, OnGenomeAnnotationImportOk, NULL);
		
		// No need for a separate OnFileExportCancel -- just need to close the
		// dialog box
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised( XtDisplay(file_dialog), XtWindow(file_dialog) );
}

void OnGenomeAnnotationImportOk(Widget w, XtPointer data, XtPointer callData)
{
 	// Copy the text from the file selection dialog to our text widget.
	char *tmp_str = NULL;
	
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
	
	try{
	
		if(ptrGP == NULL){
			throw __FILE__ ":OnGenomeAnnotationImportOk: NULL ptrGP";
		}

		if(ptrGP->import_annotation(file_name /*PTT*/) == true){
			postRedisplay();
		}
	}
	catch(const char *error){

		// Was there an error opening the file?
		AfxMessageBox(error);

		return;
	}
	catch(...){

		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to import annotation file!");

		return;
	}
}

void OnGenomeAnnotationEdit(Widget w, XtPointer data, XtPointer callData)
{
	if( (ptrGP == NULL) || (ptrGP->num_display() == 0) || (ptrGP->num_select() == 0) ){
	
		AfxMessageBox("Please open a genome before editing annotations");
		return;
	}
	
	if(annotation_dialog){

		// Update the values stored in this dialog (without creating a
		// new dialog)
		OnGenomeAnnotationUpdate();
		
		return;
	}
	
	// Build the dialog box
	Widget pane, form, button;
	
	annotation_dialog = XtVaCreatePopupShell("edit_annotation", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
		
	// Add a title to the dialog
	XtVaSetValues(annotation_dialog, 
		XmNtitle, "Edit Annotation", 
		NULL);
		
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(annotation_dialog, AnnotationDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		annotation_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
	
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 10,
		XmNwidth, 500, 
		XmNheight, 350, 
		NULL);
	
	// Create static text labels
	XtVaCreateManagedWidget("Range:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 1,
		NULL);
		
	XtVaCreateManagedWidget("Product:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 1,
		NULL);
	
	XtVaCreateManagedWidget("Note:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 6,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 7,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 1,
		NULL);
	
	XtVaCreateManagedWidget("Strand:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaCreateManagedWidget("Type:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 1,
		NULL);
	
	XtVaCreateManagedWidget("LOCUS:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 1,
		NULL);
		
	XtVaCreateManagedWidget("LOCUS TAG:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		NULL);
	
	XtVaCreateManagedWidget("EC:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 8,
		NULL);
	
	XtVaCreateManagedWidget("Accession:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
		
	XtVaCreateManagedWidget("GI:",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition,1,
		NULL);
		
	// Text entry
	annot_dialog_range_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_range_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	XtManageChild(annot_dialog_range_text);
	
	annot_dialog_product_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_product_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	XtManageChild(annot_dialog_product_text);
	
	Arg arg_list[13];
	
	XtSetArg(arg_list[0], XmNeditable, True);
	XtSetArg(arg_list[1], XmNscrollingPolicy, XmAUTOMATIC);
	XtSetArg(arg_list[2], XmNalignment, XmALIGNMENT_CENTER);
	XtSetArg(arg_list[3], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[4], XmNtopPosition, 5);
	XtSetArg(arg_list[5], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[6], XmNbottomPosition, 9);
	XtSetArg(arg_list[7], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[8], XmNleftPosition, 1);
	XtSetArg(arg_list[9], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[10], XmNrightPosition, 10);
	XtSetArg( arg_list[11], XmNvalue, (char*)annot_note_text.c_str() );
	XtSetArg(arg_list[12], XmNeditMode, XmMULTI_LINE_EDIT);
	
	annot_dialog_note_text = XmCreateScrolledText(form, 
		"not_text",
		arg_list,
		13);
	
	XtManageChild(annot_dialog_note_text);
	
	annot_dialog_locus_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_locus_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
	
	XtManageChild(annot_dialog_locus_text);
	
	annot_dialog_locus_tag_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_locus_tag_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	XtManageChild(annot_dialog_locus_tag_text);
	
	annot_dialog_ec_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_ec_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 8,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	XtManageChild(annot_dialog_locus_tag_text);
	
	annot_dialog_gi_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_gi_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	XtManageChild(annot_dialog_gi_text);
	
	annot_dialog_accession_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)annot_accession_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		NULL);
	
	XtManageChild(annot_dialog_accession_text);
	
	////////////////////////////////////////////////////////
	// Strand option
	XmString op_plus = XmStringCreateSimple("Plus");
	XmString op_minus = XmStringCreateSimple("Minus");
	
	char default_key = 'P';
	int option_set = 0;
	
	switch(annot_strand){
		case Seq_strand_plus:
			default_key = 'P';
			option_set = 0;
			break;
		case Seq_strand_minus:
			default_key = 'M';
			option_set = 1;
			break;
	};
	
	annot_dialog_strand_option = XmVaCreateSimpleOptionMenu(form, "strand_option_menu", 
		NULL, default_key /* Default key */, option_set,
		NULL /* The Callback function */,
		XmVaPUSHBUTTON, op_plus,'P', NULL, NULL, 
		XmVaPUSHBUTTON, op_minus, 'M', NULL, NULL, 
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 6,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	XmStringFree(op_plus);
	XmStringFree(op_minus);
	
	XtManageChild(annot_dialog_strand_option);
	
	////////////////////////////////////////////////////////
	// Type option
	XmString op_cds = XmStringCreateSimple("CDS");
	XmString op_gene = XmStringCreateSimple("Gene");
	XmString op_pseudogene = XmStringCreateSimple("Pseudo-gene");
	XmString op_RNA = XmStringCreateSimple("RNA");
	XmString op_tRNA = XmStringCreateSimple("tRNA");
	XmString op_misc = XmStringCreateSimple("Misc");
	XmString op_primer = XmStringCreateSimple("Primer");
	XmString op_user = XmStringCreateSimple("User");
	XmString op_none = XmStringCreateSimple("None");
	
	default_key = 'C';
	option_set = 0;
	
	switch(annot_type){
		case GeneAnnotation::CDS:
			default_key = 'C';
			option_set = 0;
			break;
		case GeneAnnotation::GENE:
			default_key = 'G';
			option_set = 1;
			break;
		case GeneAnnotation::PSEUDO_GENE:
			default_key = 'P';
			option_set = 2;
			break;
		case GeneAnnotation::RNA:
			default_key = 'R';
			option_set = 3;
			break;
		case GeneAnnotation::tRNA:
			default_key = 'T';
			option_set = 4;
			break;
		case GeneAnnotation::IMP:
			default_key = 'M';
			option_set = 5;
			break;
		case GeneAnnotation::PRIMER:
			default_key = 'I';
			option_set = 6;
			break;
		case GeneAnnotation::USER:
			default_key = 'U';
			option_set = 7;
			break;
		case GeneAnnotation::NONE:
			default_key = 'N';
			option_set = 8;
			break;
	};
	
	annot_dialog_type_option = XmVaCreateSimpleOptionMenu(form, "type_option_menu", 
		NULL, default_key /* Default key */, option_set,
		NULL /* The Callback function */,
		XmVaPUSHBUTTON, op_cds,'C', NULL, NULL, 
		XmVaPUSHBUTTON, op_gene,'G', NULL, NULL, 
		XmVaPUSHBUTTON, op_pseudogene,'P', NULL, NULL, 
		XmVaPUSHBUTTON, op_RNA,'R', NULL, NULL, 
		XmVaPUSHBUTTON, op_tRNA,'T', NULL, NULL, 
		XmVaPUSHBUTTON, op_misc,'M', NULL, NULL, 
		XmVaPUSHBUTTON, op_primer,'I', NULL, NULL, 
		XmVaPUSHBUTTON, op_user,'U', NULL, NULL, 
		XmVaPUSHBUTTON, op_none,'N', NULL, NULL, 
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
	
	XmStringFree(op_cds);
	XmStringFree(op_gene);
	XmStringFree(op_pseudogene);
	XmStringFree(op_RNA);
	XmStringFree(op_tRNA);
	XmStringFree(op_misc);
	XmStringFree(op_primer);
	XmStringFree(op_user);
	XmStringFree(op_none);
	
	XtManageChild(annot_dialog_type_option);
	
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 9,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		AnnotationDialogClose, annotation_dialog);
	
	button = XtVaCreateManagedWidget("Apply",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 9,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		OnGenomeAnnotationEditApply, annotation_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(annotation_dialog, XtGrabNone);
	
	OnGenomeAnnotationUpdate();
}

bool OnGenomeAnnotationUpdate()
{
	int gi = -1;
	
	if( (ptrGP == NULL) || (ptrGP->num_display() == 0) ){
		return false;
	}
	
	// Only update if the user has selected a single annotation
	edit_annot = ptrGP->selected_annotation();
		
	if( edit_annot.is_genome_valid() == false){
	
		AfxMessageBox("Please select a genome before editing annotations");
		return false;
	}
		
	if( edit_annot.is_annot_valid() == false){
		
		// Reset all defaults
		annot_range_text = "";
		annot_locus_text = "";
		annot_locus_tag_text = "";
		annot_product_text = "";
		annot_note_text = "";
		annot_ec_text = "";
		annot_gi_text = "";
		annot_accession_text = "";
		annot_strand = Seq_strand_plus;
		annot_type = GeneAnnotation::CDS;
	}
	else{
		annot_range_text = edit_annot.range();
		annot_locus_text = edit_annot.info(GeneAnnotation::LOCUS);
		annot_locus_tag_text = edit_annot.info(GeneAnnotation::LOCUS_TAG);
		annot_product_text = edit_annot.info(GeneAnnotation::PRODUCT);
		
		// Use a maximum line length of 50 characters
		annot_note_text = wrap_string( edit_annot.info(GeneAnnotation::NOTE), 50);
		
		annot_ec_text = edit_annot.info(GeneAnnotation::EC);
		gi = edit_annot.gi();
		annot_accession_text = edit_annot.accession(true);
		annot_type = edit_annot.type();
		annot_strand = edit_annot.strand();
	}	
	
	XtVaSetValues(annot_dialog_range_text, XmNvalue, 
			(char*)annot_range_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_locus_text, XmNvalue, 
			(char*)annot_locus_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_locus_tag_text, XmNvalue, 
			(char*)annot_locus_tag_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_product_text, XmNvalue, 
			(char*)annot_product_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_note_text, XmNvalue, 
			(char*)annot_note_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_ec_text, XmNvalue, 
			(char*)annot_ec_text.c_str(), NULL);
	
	// Only update the gi if it is valid (i.e. > 0)
	if(gi > 0){
		annot_gi_text = itoa(gi);
	}
	else{
		annot_gi_text = "";
	}
	
	XtVaSetValues(annot_dialog_gi_text, XmNvalue, 
			(char*)annot_gi_text.c_str(), NULL);
	
	XtVaSetValues(annot_dialog_accession_text, XmNvalue, 
			(char*)annot_accession_text.c_str(), NULL);
	
	///////////////////////////		
	// Update the option menus
	
	// First, get the number of buttons in this option menu
	Widget menu;
	int num_buttons;
	WidgetList button_list;
	
	// The annotation type option menu
	XtVaGetValues(annot_dialog_type_option, XmNsubMenuId, &menu, NULL);

	XtVaGetValues(menu, XmNnumChildren, &num_buttons, XmNchildren, &button_list, NULL) ;
	
	switch(annot_type){
		case GeneAnnotation::CDS:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[0], NULL);
			break;
		case GeneAnnotation::GENE:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[1], NULL);
			break;
		case GeneAnnotation::PSEUDO_GENE:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[2], NULL);
			break;
		case GeneAnnotation::RNA:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[3], NULL);
			break;
		case GeneAnnotation::tRNA:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[4], NULL);
			break;
		case GeneAnnotation::IMP:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[5], NULL);
			break;
		case GeneAnnotation::PRIMER:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[6], NULL);
			break;
		case GeneAnnotation::USER:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[7], NULL);
			break;
		case GeneAnnotation::NONE:
			XtVaSetValues(annot_dialog_type_option, XmNmenuHistory, button_list[8], NULL);
			break;
	};
	
	// The annotation strand option menu
	XtVaGetValues(annot_dialog_strand_option, XmNsubMenuId, &menu, NULL);

	XtVaGetValues(menu, XmNnumChildren, &num_buttons, XmNchildren, &button_list, NULL) ;
	
	switch(annot_strand){
		case Seq_strand_plus:
			XtVaSetValues(annot_dialog_strand_option, XmNmenuHistory, button_list[0], NULL);
			break;
		case Seq_strand_minus:
			XtVaSetValues(annot_dialog_strand_option, XmNmenuHistory, button_list[1], NULL);
			break;
	};
	
	return true;
}

void OnGenomeAnnotationEditApply(Widget w, XtPointer data, XtPointer callData)
{
	if( (ptrGP == NULL) || (ptrGP->num_display() == 0) ){
		return;
	}
	
	try{
 		// Validate the users choices
		char *buffer = NULL;

		// First process the required fields:
		XtVaGetValues(annot_dialog_range_text, XmNvalue, &buffer, NULL);
		annot_range_text = trim_space(buffer);
		
		// Update all of the non-esential values
		XtVaGetValues(annot_dialog_locus_text, XmNvalue, &buffer, NULL);
		annot_locus_text = trim_space(buffer);
		
		XtVaGetValues(annot_dialog_locus_tag_text, XmNvalue, &buffer, NULL);
		annot_locus_tag_text = trim_space(buffer);
		
		XtVaGetValues(annot_dialog_product_text, XmNvalue, &buffer, NULL);
		annot_product_text = trim_space(buffer);
		
		// Since the note can be multi-line -- remove any end of
		// line characters
		XtVaGetValues(annot_dialog_note_text, XmNvalue, &buffer, NULL);
		annot_note_text = pack_string( trim_space(buffer) );
		
		XtVaGetValues(annot_dialog_ec_text, XmNvalue, &buffer, NULL);
		annot_ec_text = trim_space(buffer);
		
		XtVaGetValues(annot_dialog_accession_text, XmNvalue, &buffer, NULL);
		annot_accession_text = trim_space(buffer);
		
		// Get the type. The number of buttons in this option menu
		Widget menu;
		int num_buttons;
		WidgetList button_list;
		Widget selected_button;
		
		// The annotation type option menu
		XtVaGetValues(annot_dialog_type_option, XmNsubMenuId, &menu, NULL);
		XtVaGetValues(annot_dialog_type_option, XmNmenuHistory, &selected_button, NULL);
		XtVaGetValues(menu, XmNnumChildren, &num_buttons, XmNchildren, &button_list, NULL) ;
		
		if(num_buttons != 9){
			throw "Couldn't parse type options menu -- update button count";
		}
		
		if(selected_button == button_list[0]){
			annot_type = GeneAnnotation::CDS;
		}
		
		if(selected_button == button_list[1]){
			annot_type = GeneAnnotation::GENE;
		}
		
		if(selected_button == button_list[2]){
			annot_type = GeneAnnotation::PSEUDO_GENE;
		}
		
		if(selected_button == button_list[3]){
			annot_type = GeneAnnotation::RNA;
		}
		
		if(selected_button == button_list[4]){
			annot_type = GeneAnnotation::tRNA;
		}
		
		if(selected_button == button_list[5]){
			annot_type = GeneAnnotation::IMP;
		}
		
		if(selected_button == button_list[6]){
			annot_type = GeneAnnotation::PRIMER;
		}
		
		if(selected_button == button_list[7]){
			annot_type = GeneAnnotation::USER;
		}
		
		if(selected_button == button_list[8]){
			annot_type = GeneAnnotation::NONE;
		}
		
		// Get the strand
		XtVaGetValues(annot_dialog_strand_option, XmNsubMenuId, &menu, NULL);
		XtVaGetValues(annot_dialog_strand_option, XmNmenuHistory, &selected_button, NULL);
		XtVaGetValues(menu, XmNnumChildren, &num_buttons, XmNchildren, &button_list, NULL) ;
		
		if(num_buttons != 2){
			throw "Couldn't parse strand options menu -- update button count";
		}
		
		if(selected_button == button_list[0]){
			annot_strand = Seq_strand_plus;
		}
		
		if(selected_button == button_list[1]){
			annot_strand = Seq_strand_minus;
		}
		
		// Validate gi
		XtVaGetValues(annot_dialog_gi_text, XmNvalue, &buffer, NULL);
		annot_gi_text = trim_space(buffer);
		
		int gi = -1;
		
		if( !annot_gi_text.empty() ){
		
			if(integer(annot_gi_text) == false){
				throw "gi must be a positive  integer";
			}
			
			gi = atoi( annot_gi_text.c_str() );
		}
		
		// Save the users choices
		edit_annot.range(annot_range_text);
		
		edit_annot.type(annot_type);
		edit_annot.strand(annot_strand);
		edit_annot.info(GeneAnnotation::LOCUS, annot_locus_text);
		edit_annot.info(GeneAnnotation::LOCUS_TAG, annot_locus_tag_text);
		edit_annot.info(GeneAnnotation::PRODUCT, annot_product_text);
		edit_annot.info(GeneAnnotation::NOTE, annot_note_text);
		edit_annot.info(GeneAnnotation::EC, annot_ec_text);
		
		SeqIdPtr sip = NULL;
		
		if(gi > 0){
		
			sip = ValNodeNew(NULL);
			sip->choice = SEQID_GI;
			sip->data.intvalue = gi;
		}
		
		if(annot_accession_text != ""){
			
			if(sip == NULL){
			
				if(annot_accession_text.find('|') != string::npos){
					// Yes
					sip = SeqIdParse ( (char*)annot_accession_text.c_str() );
				}
				else{
					// No
					sip = SeqIdParse( (char*)string("gb|" + annot_accession_text).c_str() );
				}
				
				if(sip == NULL){
					throw "Unable to write accession";
				}
			}
			else{
				if(annot_accession_text.find('|') != string::npos){
					// Yes
					sip->next = SeqIdParse ( (char*)annot_accession_text.c_str() );
				}
				else{
					// No
					sip->next = SeqIdParse( (char*)string("gb|" + annot_accession_text).c_str() );
				}
				
				if(sip->next == NULL){
					throw "Unable to write accession";
				}
			}
		}
		
		if(sip){
			// Only update the seqid if the user has provided the info
			edit_annot.seqid(sip);
			sip = SeqIdSetFree(sip);
		}
		
		ptrGP->update_annotation(edit_annot);
		
		postRedisplay();
	}
	catch(const char *error){
		AfxMessageBox(error);
	}
	catch(...){
		AfxMessageBox("Error editing annotation");
	}
}

void AnnotationDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(annotation_dialog);
	
	// Allow the user to open another dialog at a later date
	annotation_dialog = NULL;
}
	
void OnGenomeHide(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeHide: NULL ptrGP";
	}
		
	if(ptrGP->hide_selection() == true){
		postRedisplay();
	}
}

void OnGenomeAnnotationHide(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeAnnotationHide: NULL ptrGP";
	}
		
	if(ptrGP->hide_selection_annotation() == true){
		postRedisplay();
	}
}

void OnGenomeShowAll(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeShowAll: NULL ptrGP";
	}
		
	if(ptrGP->show_all() == true){
		postRedisplay();
	}
}

void OnGenomeShowAnnotation(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeShowAnnotation: NULL ptrGP";
	}
		
	if(ptrGP->show_annotation() == true){
		postRedisplay();
	}
}

void OnGenomeGC(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeGC: NULL ptrGP";
	}
		
	if(ptrGP->toggle_gc_selection() == true){
		postRedisplay();
	}
}

void OnGenomeAT(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeAT: NULL ptrGP";
	}
		
	if(ptrGP->toggle_at_selection() == true){
		postRedisplay();
	}
}

void OnGenomeGCSkew(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeGCSkew: NULL ptrGP";
	}
		
	if(ptrGP->toggle_gc_skew_selection() == true){
		postRedisplay();
	}
}

void OnGenomeATSkew(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeATSkew: NULL ptrGP";
	}
		
	if(ptrGP->toggle_at_skew_selection() == true){
		postRedisplay();
	}
}

void OnGenomeKetoSkew(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeKetoSkew: NULL ptrGP";
	}
		
	if(ptrGP->toggle_keto_skew_selection() == true){
		postRedisplay();
	}
}

void OnGenomePurineSkew(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomePurineSkew: NULL ptrGP";
	}
		
	if(ptrGP->toggle_purine_skew_selection() == true){
		postRedisplay();
	}
}

void OnGenomeTm(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeTm: NULL ptrGP";
	}
		
	if(ptrGP->toggle_tm_selection() == true){
		postRedisplay();
	}
}

void OnGenomeUser(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeUser: NULL ptrGP";
	}
		
	if(ptrGP->toggle_genome_plot_selection() == true){
		postRedisplay();
	}
}

void OnGenomeConservation(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeConservation: NULL ptrGP";
	}
		
	if(ptrGP->toggle_conservation_plot() == true){
		postRedisplay();
	}
}

void OnGenomeMoveUp(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeMoveUp: NULL ptrGP";
	}
		
	if(ptrGP->shift_selection_up() == true){
		postRedisplay();
	}
}

void OnGenomeMoveDown(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeMoveDown: NULL ptrGP";
	}
		
	if(ptrGP->shift_selection_down() == true){
		postRedisplay();
	}
}

void OnGenomeShowMore(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeShowMore: NULL ptrGP";
	}
		
	if(ptrGP->key_plus() == true){
		postRedisplay();
	}
}
void OnGenomeShowLess(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeShowLess: NULL ptrGP";
	}
		
	if(ptrGP->key_minus() == true){
		postRedisplay();
	}
}

void OnGenomeInvert(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnGenomeInvert: NULL ptrGP";
	}
		
	if(ptrGP->complement_selection() == true){
		postRedisplay();
	}
}

void OnSequenceDNA(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnSequenceDNA: NULL ptrGP";
	}
	
	string info = ptrGP->selected_seq_as_dna();
	
	if(info != ""){
		InfoDialog(XtParent(w), (char*)info.c_str(), NULL);
	}
}

void OnSequenceProtein(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnSequenceProtein: NULL ptrGP";
	}
	
	string info = ptrGP->selected_seq_as_protein();
	
	if(info != ""){
		InfoDialog(XtParent(w), (char*)info.c_str(), NULL);
	}
}

void OnColorBackground(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_BACKGROUND_R, 
					    GV_DEFAULT_BACKGROUND_G, 
					    GV_DEFAULT_BACKGROUND_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorBackgroundOk);
}

void OnColorBackgroundOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorBackgroundOk: NULL ptrGP";
	}
	
	ptrGP->background_color(color_dialog_rgb[0], 
					    color_dialog_rgb[1],
					    color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorSelection(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_SELECT_R, 
					    GV_DEFAULT_SELECT_G, 
					    GV_DEFAULT_SELECT_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorSelectionOk);
}

void OnColorSelectionOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorSelectionOk: NULL ptrGP";
	}
	
	ptrGP->select_color(color_dialog_rgb[0], 
					color_dialog_rgb[1],
					color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorText(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_TEXT_R, 
					    GV_DEFAULT_TEXT_G, 
					    GV_DEFAULT_TEXT_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorTextOk);
}

void OnColorTextOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorTextOk: NULL ptrGP";
	}
	
	ptrGP->text_color(color_dialog_rgb[0], 
				   color_dialog_rgb[1],
				   color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorGene(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_GENE_R, 
					    GV_DEFAULT_GENE_G, 
					    GV_DEFAULT_GENE_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorGeneOk);
}

void OnColorGeneOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorGeneOk: NULL ptrGP";
	}
	
	ptrGP->gene_color(color_dialog_rgb[0], 
				   color_dialog_rgb[1],
				   color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorPseudoGene(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_PSEUDO_GENE_R, 
					    GV_DEFAULT_PSEUDO_GENE_G, 
					    GV_DEFAULT_PSEUDO_GENE_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorPseudoGeneOk);
}

void OnColorPseudoGeneOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorPseudoGeneOk: NULL ptrGP";
	}
	
	ptrGP->pseudo_gene_color(color_dialog_rgb[0], 
				   color_dialog_rgb[1],
				   color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorCDS(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_CDS_R, 
					    GV_DEFAULT_CDS_G, 
					    GV_DEFAULT_CDS_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorCDSOk);
}

void OnColorCDSOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorCDSOk: NULL ptrGP";
	}
	
	ptrGP->cds_color(color_dialog_rgb[0], 
				  color_dialog_rgb[1],
				  color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorAnnot(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_ANNOT_R, 
					    GV_DEFAULT_ANNOT_G, 
					    GV_DEFAULT_ANNOT_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorAnnotOk);
}

void OnColorAnnotOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorAnnotOk: NULL ptrGP";
	}
	
	ptrGP->annot_color(color_dialog_rgb[0], 
				    color_dialog_rgb[1],
				    color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorRNA(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_RNA_R, 
					    GV_DEFAULT_RNA_G, 
					    GV_DEFAULT_RNA_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorRNAOk);
}

void OnColorRNAOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorRNAOk: NULL ptrGP";
	}
	
	ptrGP->rna_color(color_dialog_rgb[0], 
				  color_dialog_rgb[1],
				  color_dialog_rgb[2]);

	postRedisplay();
}

void OnColorIntergenic(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_INTERGENIC_R, 
					    GV_DEFAULT_INTERGENIC_G, 
					    GV_DEFAULT_INTERGENIC_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorIntergenicOk);
}

void OnColorIntergenicOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorIntergenicOk: NULL ptrGP";
	}
	
	ptrGP->intergenic_color(color_dialog_rgb[0], 
				  	    color_dialog_rgb[1],
				  	    color_dialog_rgb[2]);

	postRedisplay();
}


void OnColorCustom(Widget w, XtPointer data, XtPointer callData)
{
	if(color_dialog){
		return;
	}
	
	unsigned char rgb[3] = {GV_DEFAULT_CUSTOM_R, 
					    GV_DEFAULT_CUSTOM_G, 
					    GV_DEFAULT_CUSTOM_B};
		
	ColorDialog(XtParent(w), rgb, (XtPointer)OnColorCustomOk);
}

void OnColorCustomOk(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OnColorCustomOk: NULL ptrGP";
	}
	
	ptrGP->custom_color(color_dialog_rgb[0], 
				  	    color_dialog_rgb[1],
				  	    color_dialog_rgb[2]);

	postRedisplay();

}

// Same name as the windows message box saves precious key strokes
void AfxMessageBox(const string &m_error)
{		
	XmString text = XmStringCreateSimple((char*)m_error.c_str());
		
	Widget errorDialog = XmCreateErrorDialog(toplevel, 
		"error", NULL, 0);
   
	// Add a title to the dialog
	XtVaSetValues(XtParent(errorDialog), XmNtitle, 
		"Error!", NULL);
		
	XtUnmanageChild(XmMessageBoxGetChild(errorDialog,
		XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild(XmMessageBoxGetChild(errorDialog,
		XmDIALOG_HELP_BUTTON));
    	
	XtVaSetValues(errorDialog, XmNmessageString, text, NULL);
	XmStringFree(text);
	XtManageChild(errorDialog);
		
	XtPopup(XtParent(errorDialog), XtGrabNone);
}

// Utility Function to get the parent of the widget tree
Widget GetTopShell(Widget w)
{
	while(w && !XtIsWMShell(w))
		w = XtParent(w);
		
	return w;
}

void ColorDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one color dialog is allowed at any one time
	if(color_dialog){
		return;
	}
	
	color_dialog_rgb[0] = ((unsigned char *)data)[0];
	color_dialog_rgb[1] = ((unsigned char *)data)[1];
	color_dialog_rgb[2] = ((unsigned char *)data)[2];
	
	color_callData = callData;
	
	Widget pane, form, button, scale, label;
	
	color_dialog = XtVaCreatePopupShell("Color", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(color_dialog, 
		XmNtitle, "Set Color", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(color_dialog, ColorDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		color_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 6,
		NULL);
	
	label = XtVaCreateManagedWidget("Red",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
		
	label = XtVaCreateManagedWidget("Green",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		NULL);
		
	label = XtVaCreateManagedWidget("Blue",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	scale = XtVaCreateManagedWidget("scaleR", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmVERTICAL,
		XmNminimum, 0,
		XmNmaximum, 255,
		XmNshowValue, True,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, color_dialog_rgb[0], NULL);
	XtAddCallback(scale, XmNdragCallback, ColorDialogR, NULL);
	
	scale = XtVaCreateManagedWidget("scaleG", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmVERTICAL,
		XmNminimum, 0,
		XmNmaximum, 255,
		XmNshowValue, True,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, color_dialog_rgb[1], NULL);
	XtAddCallback(scale, XmNdragCallback, ColorDialogG, NULL);
	
	scale = XtVaCreateManagedWidget("scaleB", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmVERTICAL,
		XmNminimum, 0,
		XmNmaximum, 255,
		XmNshowValue, True,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, color_dialog_rgb[2], NULL);
	XtAddCallback(scale, XmNdragCallback, ColorDialogB, NULL);
		
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 5,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 6,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		ColorDialogClose, color_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(color_dialog, XtGrabNone);
}

void ColorDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(color_dialog);
	
	// Allow the user to open another dialog at a later date
	color_dialog = NULL;
	color_callData = NULL;
}

void ColorDialogR(Widget w, XtPointer data, XtPointer callData)
{	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	
	color_dialog_rgb[0] = scale_data->value;
	
	// Update the color in real time
	((void (*)(_WidgetRec*, void*, void*))color_callData)(w, data, callData);
}

void ColorDialogG(Widget w, XtPointer data, XtPointer callData)
{
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	
	color_dialog_rgb[1] = scale_data->value;
	
	// Update the color in real time
	((void (*)(_WidgetRec*, void*, void*))color_callData)(w, data, callData);
}

void ColorDialogB(Widget w, XtPointer data, XtPointer callData)
{
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	
	color_dialog_rgb[2] = scale_data->value;
	
	// Update the color in real time
	((void (*)(_WidgetRec*, void*, void*))color_callData)(w, data, callData);
}

void InfoDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one info dialog is allowed at any one time.
	// However, if an info dialog is already open, change the
	// displayed text to match the multiline string stored in data.
	if(info_dialog){
		// Update the string here
		XtVaSetValues(info_dialog_text, XmNvalue, (char*)data, NULL);
		return;
	}
		
	Widget pane, form, button;
	
	info_dialog = XtVaCreatePopupShell("Info", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(info_dialog, 
		XmNtitle, "Annotation Information", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(info_dialog, InfoDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		info_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 5,
		NULL);
	
	Arg arg_list[13];
	
	XtSetArg(arg_list[0], XmNeditable, False);
	XtSetArg(arg_list[1], XmNscrollingPolicy, XmAUTOMATIC);
	XtSetArg(arg_list[2], XmNalignment, XmALIGNMENT_CENTER);
	XtSetArg(arg_list[3], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[4], XmNtopPosition, 0);
	XtSetArg(arg_list[5], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[6], XmNbottomPosition, 4);
	XtSetArg(arg_list[7], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[8], XmNleftPosition, 0);
	XtSetArg(arg_list[9], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[10], XmNrightPosition, 5);
	XtSetArg(arg_list[11], XmNvalue, (char*)data);
	XtSetArg(arg_list[12], XmNeditMode, XmMULTI_LINE_EDIT);
	
	info_dialog_text = XmCreateScrolledText(form, 
		"info_text",
		arg_list,
		13);
	
	XtManageChild(info_dialog_text);
	
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	// Attach a user supplied "OK" call back
	XtAddCallback(button, XmNactivateCallback, 
		InfoDialogClose, info_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(info_dialog, XtGrabNone);
}

void InfoDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(info_dialog);
	
	// Allow the user to open another dialog at a later date
	info_dialog = NULL;
}

void FindDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one find dialog is allowed at any one time.
	if(find_dialog){
		return;
	}
		
	Widget pane, form, button, option;
	
	find_dialog = XtVaCreatePopupShell("find", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(find_dialog, 
		XmNtitle, "Search", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(find_dialog, FindDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		find_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 9,
		XmNwidth, 450, 
		XmNheight, 300, 
		NULL);
	
	// Create static text label.
	XtVaCreateManagedWidget("Search for",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		NULL);
		
	// Add the option menu
	XmString op_name =			XmStringCreateSimple("Gene Name/Tag");
	XmString op_gi =			XmStringCreateSimple("GI");
	XmString op_base = 			XmStringCreateSimple("Base Number");
	XmString op_key_text = 		XmStringCreateSimple("Annotation keyword(s)");
	XmString op_dna = 			XmStringCreateSimple("DNA Sequence");
	XmString op_genome_name = 	XmStringCreateSimple("Genome Name");
	
	char default_key = 'N';
	int option_set = 0;
	
	switch(search_key){
		case GenomePage::SEARCH_KEY_NAME:
			default_key = 'N';
			option_set = 0;
			break;
		case GenomePage::SEARCH_KEY_GI:
			default_key = 'G';
			option_set = 1;
			break;
		case GenomePage::SEARCH_KEY_BASE:
			default_key = 'B';
			option_set = 2;
			break;
		case GenomePage::SEARCH_KEY_TEXT:
			default_key = 'K';
			option_set = 3;
			break;
		case GenomePage::SEARCH_KEY_DNA:
			default_key = 'D';
			option_set = 4;
			break;
		case GenomePage::SEARCH_KEY_GENOME_NAME:
			default_key = 'O';
			option_set = 5;
			break;
	};
	
	option = XmVaCreateSimpleOptionMenu(form, "option_menu", 
		NULL, default_key /* Default key */, option_set,
		FindDialogSearchKey /* The Callback function */,
		XmVaPUSHBUTTON, op_name,'N', NULL, NULL, 
		XmVaPUSHBUTTON, op_gi, 'G', NULL, NULL, 
		XmVaPUSHBUTTON, op_base, 'B', NULL, NULL, 
		XmVaPUSHBUTTON, op_key_text, 'K', NULL, NULL, 
		XmVaPUSHBUTTON, op_dna, 'D', NULL, NULL, 
		XmVaPUSHBUTTON, op_genome_name, 'O', NULL, NULL, 
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	XmStringFree(op_name);
	XmStringFree(op_gi);
	XmStringFree(op_base);
	XmStringFree(op_key_text);
	XmStringFree(op_dna);
	XmStringFree(op_genome_name);
	
	XtManageChild(option);
		
	find_dialog_search_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)search_text.c_str(),
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	XtManageChild(find_dialog_search_text);
	
	Arg arg_list[13];
	
	XtSetArg(arg_list[0], XmNeditable, False);
	XtSetArg(arg_list[1], XmNscrollingPolicy, XmAUTOMATIC);
	XtSetArg(arg_list[2], XmNalignment, XmALIGNMENT_CENTER);
	XtSetArg(arg_list[3], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[4], XmNtopPosition, 4);
	XtSetArg(arg_list[5], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[6], XmNbottomPosition, 8);
	XtSetArg(arg_list[7], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[8], XmNleftPosition, 0);
	XtSetArg(arg_list[9], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[10], XmNrightPosition, 9);
	XtSetArg(arg_list[11], XmNvalue, (char*)search_results.c_str());
	XtSetArg(arg_list[12], XmNeditMode, XmMULTI_LINE_EDIT);
	
	find_dialog_results_text = XmCreateScrolledText(form, 
		"results",
		arg_list,
		13);
	
	XtManageChild(find_dialog_results_text);
	
	button = XtVaCreateManagedWidget("Find",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		FindDialogSearch, find_dialog);
		
	button = XtVaCreateManagedWidget("Find again",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		FindDialogSearchAgain, find_dialog);
	
	button = XtVaCreateManagedWidget("Annotate",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		FindDialogAnnotate, find_dialog);
		
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		FindDialogClose, find_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(find_dialog, XtGrabNone);
}

void FindDialogSearch(Widget w, XtPointer data, XtPointer callData)
{
	// Only one search can run at a time!
	static bool is_searching = false;
	
	if(is_searching){
		return;
	}
	
	is_searching = true;
	
	WaitCursor(find_dialog);
	
	char *buffer;
	
	// Load the search key and search string from the dialog and
	// start searching
	
	XtVaGetValues(find_dialog_search_text, XmNvalue, &buffer, NULL);
	search_text = buffer;
	
	try{
		if(ptrGP == NULL){
			throw __FILE__ ":FindDialogSearch: NULL ptrGP";
		}
	
		if(ptrGP->validate_search_key(search_key, search_text) == false){

			throw "Invalid gene LOCUS/LOCUS TAG";
		}

		// Search all genomes
		search_results = ptrGP->search(search_key, search_text);

		// Display the search results
		XtVaSetValues(find_dialog_results_text, XmNvalue, 
			(char*)search_results.c_str(), NULL);
		
		postRedisplay();
	}
	catch(const char *error){
		ArrowCursor(find_dialog);
		AfxMessageBox(error);
		is_searching = false;
	}
	catch(...){
		ArrowCursor(find_dialog);
		AfxMessageBox("Unknown error occured while searching");
		is_searching = false;
	}
	
	ArrowCursor(find_dialog);
	
	is_searching = false;
}

void FindDialogSearchAgain(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":FindDialogSearchAgain: NULL ptrGP";
	}
		
	ptrGP->search_next();

	postRedisplay();
}

void FindDialogAnnotate(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":FindDialogAnnotate: NULL ptrGP";
	}
	
	bool ret = ptrGP->annotate_selection();
	
	if(ret == false){
		AfxMessageBox("There must be at least one selected sequence region to annotate");
	}
	else{
		// Show the new annotation(s)
		postRedisplay();
	}
}


void FindDialogSearchKey(Widget w, XtPointer data, XtPointer callData)
{
	char *type_name = XtName(w);
	
	// Note that XtName(widget) returns a normal text string
	// "button_0", "button_1", "button_2" etc, depending on the
	// item selected.
	if(strcmp(type_name, "button_0") == 0){
		search_key = GenomePage::SEARCH_KEY_NAME;
		return;
	}
	
	if(strcmp(type_name, "button_1") == 0){
		search_key = GenomePage::SEARCH_KEY_GI;
		return;
	}
	
	if(strcmp(type_name, "button_2") == 0){
		search_key = GenomePage::SEARCH_KEY_BASE;
		return;
	}
	
	if(strcmp(type_name, "button_3") == 0){
		search_key = GenomePage::SEARCH_KEY_TEXT;
		return;
	}
	
	if(strcmp(type_name, "button_4") == 0){
		search_key = GenomePage::SEARCH_KEY_DNA;
		return;
	}
	
	if(strcmp(type_name, "button_5") == 0){
		search_key = GenomePage::SEARCH_KEY_GENOME_NAME;
		return;
	}
}

void FindDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(find_dialog);
	
	// Allow the user to open another dialog at a later date
	find_dialog = NULL;
}

void HybridDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one hybridize dialog is allowed at any one time.
	if(hybrid_dialog){
		return;
	}
		
	Widget pane, form, button;
	
	hybrid_dialog = XtVaCreatePopupShell("Info", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(hybrid_dialog, 
		XmNtitle, "Hybridization", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(hybrid_dialog, HybridDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		hybrid_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 90,
		XmNwidth, 650, 
		XmNheight, 300, 
		NULL);
		
	// Create static text labels
	XtVaCreateManagedWidget("[Strand] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 15,
		NULL);
	
	XtVaCreateManagedWidget("M, [Na+] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 26,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 35,
		NULL);
	
	XtVaCreateManagedWidget("M, Hybridize if Tm > ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 46,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 65,
		NULL);
	
	XtVaCreateManagedWidget("C",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 76,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 80,
		NULL);
	
	XtVaCreateManagedWidget("primer (1) 5'-",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 13,
		NULL);
	
	XtVaCreateManagedWidget("-3'",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 85,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	XtVaCreateManagedWidget("primer (2) 5'-",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 13,
		NULL);
	
	XtVaCreateManagedWidget("-3'",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 85,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	XtVaCreateManagedWidget("<= amplicon length <=",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 10,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 30,
		NULL);
		
	XtVaCreateManagedWidget("Require an exact 3' match of ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 40,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 70,
		NULL);
	
	XtVaCreateManagedWidget("bases",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 80,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	XtVaCreateManagedWidget("probe 5'-",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 40,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 50,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 13,
		NULL);
	
	XtVaCreateManagedWidget("-3'",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 40,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 50,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 85,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	// Create editable text fields for user input	
	hybrid_dialog_strand_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_strand_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 15,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 25,
		NULL);
	
	hybrid_dialog_salt_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_salt_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 35,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 45,
		NULL);
		
	hybrid_dialog_tm_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_tm_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 65,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 75,
		NULL);
	
	hybrid_dialog_primer1_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_primer1_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 13,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 85,
		NULL);
	
	hybrid_dialog_primer2_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_primer2_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 13,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 85,
		NULL);
	
	hybrid_dialog_min_amplicon_len_text =
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_min_amplicon_len_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	hybrid_dialog_max_amplicon_len_text =
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_max_amplicon_len_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 30,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 40,
		NULL);
			
	hybrid_dialog_exact_text =
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_exact_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 70,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 80,
		NULL);
	
	hybrid_dialog_probe_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)hybrid_probe_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 40,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 50,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 13,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 85,
		NULL);
	
	Arg arg_list[13];
	
	XtSetArg(arg_list[0], XmNeditable, False);
	XtSetArg(arg_list[1], XmNscrollingPolicy, XmAUTOMATIC);
	XtSetArg(arg_list[2], XmNalignment, XmALIGNMENT_CENTER);
	XtSetArg(arg_list[3], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[4], XmNtopPosition, 50);
	XtSetArg(arg_list[5], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[6], XmNbottomPosition, 80);
	XtSetArg(arg_list[7], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[8], XmNleftPosition, 0);
	XtSetArg(arg_list[9], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[10], XmNrightPosition, 90);
	XtSetArg(arg_list[11], XmNvalue, (char*)hybrid_results_text.c_str());
	XtSetArg(arg_list[12], XmNeditMode, XmMULTI_LINE_EDIT);
	
	hybrid_dialog_results_text = XmCreateScrolledText(form, 
		"hybrid_results",
		arg_list,
		13);
	
	XtManageChild(hybrid_dialog_results_text);
	
	button = XtVaCreateManagedWidget("Search with Primers",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 20,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		HybridDialogSearchPrimers, hybrid_dialog);
	
	button = XtVaCreateManagedWidget("Search with Probe",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 20,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 40,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		HybridDialogSearchProbe, hybrid_dialog);
	
	button = XtVaCreateManagedWidget("Show Next",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 40,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 60,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		HybridDialogShowNext, hybrid_dialog);
	
	button = XtVaCreateManagedWidget("Annotate",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 60,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 80,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		HybridDialogAnnotate, hybrid_dialog);
		
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 80,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		HybridDialogClose, hybrid_dialog);
			
	XtManageChild(form);
	
	XtManageChild(pane);
	
		
	XtPopup(hybrid_dialog, XtGrabNone);
}

void HybridDialogSearchPrimers(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":HybridDialogSearchPrimers: NULL ptrGP";
	}
	
	// Only one search can run at a time!
	static bool is_working = false;
	
	if(is_working){
		return;
	}
	
	is_working = true;
	
	WaitCursor(hybrid_dialog);
	
	// Perform a primer based search
	int exact_match;
	int min_amplicon_len;
	int max_amplicon_len;
	
	char *buffer;
	
	XtVaGetValues(hybrid_dialog_primer1_text, XmNvalue, &buffer, NULL);
	hybrid_primer1_text = buffer;
	
	XtVaGetValues(hybrid_dialog_primer2_text, XmNvalue, &buffer, NULL);
	hybrid_primer2_text = buffer;
	
	XtVaGetValues(hybrid_dialog_strand_text, XmNvalue, &buffer, NULL);
	hybrid_strand_text = buffer;
	
	XtVaGetValues(hybrid_dialog_salt_text, XmNvalue, &buffer, NULL);
	hybrid_salt_text = buffer;
	
	XtVaGetValues(hybrid_dialog_tm_text, XmNvalue, &buffer, NULL);
	hybrid_tm_text = buffer;
	
	XtVaGetValues(hybrid_dialog_min_amplicon_len_text, XmNvalue, &buffer, NULL);
	hybrid_min_amplicon_len_text = buffer;
	
	min_amplicon_len = atoi(buffer);
	
	XtVaGetValues(hybrid_dialog_max_amplicon_len_text, XmNvalue, &buffer, NULL);
	hybrid_max_amplicon_len_text = buffer;
	
	max_amplicon_len = atoi(buffer);
	
	XtVaGetValues(hybrid_dialog_exact_text, XmNvalue, &buffer, NULL);
	hybrid_exact_text = buffer;
	
	exact_match = atoi(buffer);
	
	if(exact_match < 0){
		ArrowCursor(hybrid_dialog);
		
		AfxMessageBox("Please enter 0 or more bases for the exact 3' match");
		
		is_working = false;
		
		return;
	}
	
	if(min_amplicon_len < 0){
		ArrowCursor(hybrid_dialog);
		
		AfxMessageBox("Please enter 0 or more bases for the min amplicon len");
		
		is_working = false;
		
		return;
	}
	
	if(max_amplicon_len < 0){
		ArrowCursor(hybrid_dialog);
		
		AfxMessageBox("Please enter 0 or more bases for the max amplicon len");
		
		is_working = false;
		
		return;
	}
	
	if(max_amplicon_len < min_amplicon_len){
		ArrowCursor(hybrid_dialog);
		
		AfxMessageBox("Max amplicon length must be greater than the min length");
		
		is_working = false;
		
		return;
	}
	
	try{
				
		hybrid_results_text = ptrGP->search_primers(
			hybrid_primer1_text, 
			hybrid_primer2_text, 
			atof(hybrid_salt_text.c_str()), 
			atof(hybrid_strand_text.c_str()), 
			atof(hybrid_tm_text.c_str()),
			exact_match,
			min_amplicon_len,
			max_amplicon_len
			);
	}
	catch(const char *error){
		hybrid_results_text = error;
	}
	
	// Display the results
	XtVaSetValues(hybrid_dialog_results_text, XmNvalue, 
		(char*)hybrid_results_text.c_str(), NULL);
	
	ArrowCursor(hybrid_dialog);
	
	// Update the display to highlight any selected annotations
	postRedisplay();
	
	is_working = false;
}

void HybridDialogSearchProbe(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":HybridDialogSearchProbe: NULL ptrGP";
	}
	
	// Only one search can run at a time!
	static bool is_working = false;
	
	if(is_working){
		return;
	}
	
	is_working = true;
	
	WaitCursor(hybrid_dialog);
				
	// Perform a probe based search
	char *buffer;
	
	XtVaGetValues(hybrid_dialog_probe_text, XmNvalue, &buffer, NULL);
	hybrid_probe_text = buffer;
		
	XtVaGetValues(hybrid_dialog_strand_text, XmNvalue, &buffer, NULL);
	hybrid_strand_text = buffer;
	
	XtVaGetValues(hybrid_dialog_salt_text, XmNvalue, &buffer, NULL);
	hybrid_salt_text = buffer;
	
	XtVaGetValues(hybrid_dialog_tm_text, XmNvalue, &buffer, NULL);
	hybrid_tm_text = buffer;
		
	try{
				
		hybrid_results_text = ptrGP->search_probe(
			hybrid_probe_text, 
			atof(hybrid_salt_text.c_str()), 
			atof(hybrid_strand_text.c_str()), 
			atof(hybrid_tm_text.c_str()));
	}
	catch(const char *error){
		hybrid_results_text = error;
	}
	
	// Display the results
	XtVaSetValues(hybrid_dialog_results_text, XmNvalue, 
		(char*)hybrid_results_text.c_str(), NULL);
	
	ArrowCursor(hybrid_dialog);
	
	// Update the display to highlight any selected annotations
	postRedisplay();
	
	is_working = false;
}

void HybridDialogShowNext(Widget w, XtPointer data, XtPointer callData)
{
	FindDialogSearchAgain(w, data, callData);
}

void HybridDialogAnnotate(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":HybridDialogAnnotate: NULL ptrGP";
	}
	
	bool ret = ptrGP->annotate_selection();
	
	if(ret == false){
		AfxMessageBox("There must be at least one selected sequence region to annotate");
	}
	else{
		// Show the new annotation(s)
		postRedisplay();
	}
}

void HybridDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(hybrid_dialog);
	
	// Allow the user to open another dialog at a later date
	hybrid_dialog = NULL;
}

void PadlockDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one padlock dialog is allowed at any one time.
	if(padlock_dialog){
		return;
	}
		
	Widget pane, form, button;
	
	padlock_dialog = XtVaCreatePopupShell("padlock_dialog", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(padlock_dialog, 
		XmNtitle, "Padlock Probe Hybridization", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(padlock_dialog, PadlockDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		padlock_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 90,
		XmNwidth, 650, 
		XmNheight, 300, 
		NULL);
		
	// Create static text labels
	XtVaCreateManagedWidget("[Strand] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 15,
		NULL);
	
	XtVaCreateManagedWidget("M, [Na+] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 26,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 35,
		NULL);
	
	XtVaCreateManagedWidget("M, Hybridize if Tm > ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 46,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 65,
		NULL);
	
	XtVaCreateManagedWidget("C",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 76,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 80,
		NULL);
	
	XtVaCreateManagedWidget("probe (1) 5'-",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 13,
		NULL);
	
	XtVaCreateManagedWidget("-3'",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 85,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	XtVaCreateManagedWidget("probe (2) 5'-",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 13,
		NULL);
	
	XtVaCreateManagedWidget("-3'",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 85,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	XtVaCreateManagedWidget("Require an exact match of ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 30,
		NULL);
	
	XtVaCreateManagedWidget("bases about ligation site",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 40,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 90,
		NULL);
	
	// Create editable text fields for user input	
	padlock_dialog_strand_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_strand_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 15,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 25,
		NULL);
	
	padlock_dialog_salt_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_salt_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 35,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 45,
		NULL);
		
	padlock_dialog_tm_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_tm_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 65,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 75,
		NULL);
	
	padlock_dialog_primer1_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_primer1_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 20,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 13,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 85,
		NULL);
	
	padlock_dialog_primer2_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_primer2_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 20,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 30,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 13,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 85,
		NULL);
			
	padlock_dialog_exact_text =
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)padlock_exact_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 30,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 40,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 30,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 40,
		NULL);
	
	Arg arg_list[13];
	
	XtSetArg(arg_list[0], XmNeditable, False);
	XtSetArg(arg_list[1], XmNscrollingPolicy, XmAUTOMATIC);
	XtSetArg(arg_list[2], XmNalignment, XmALIGNMENT_CENTER);
	XtSetArg(arg_list[3], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[4], XmNtopPosition, 40);
	XtSetArg(arg_list[5], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[6], XmNbottomPosition, 80);
	XtSetArg(arg_list[7], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[8], XmNleftPosition, 0);
	XtSetArg(arg_list[9], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[10], XmNrightPosition, 90);
	XtSetArg(arg_list[11], XmNvalue, (char*)padlock_results_text.c_str());
	XtSetArg(arg_list[12], XmNeditMode, XmMULTI_LINE_EDIT);
	
	padlock_dialog_results_text = XmCreateScrolledText(form, 
		"padlock_results",
		arg_list,
		13);
	
	XtManageChild(padlock_dialog_results_text);
	
	button = XtVaCreateManagedWidget("Search with probes",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 10,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 30,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		PadlockDialogSearchPrimers, padlock_dialog);
	
	button = XtVaCreateManagedWidget("Show Next",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 30,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 50,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		PadlockDialogShowNext, padlock_dialog);
	
	button = XtVaCreateManagedWidget("Annotate",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 50,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 70,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		PadlockDialogAnnotate, padlock_dialog);
		
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 80,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 90,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 70,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 80,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		PadlockDialogClose, padlock_dialog);
			
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(padlock_dialog, XtGrabNone);
}

void PadlockDialogSearchPrimers(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":PadlockDialogSearchPrimers: NULL ptrGP";
	}
	
	// Only one search can run at a time!
	static bool is_working = false;
	
	if(is_working){
		return;
	}
	
	is_working = true;
	
	WaitCursor(padlock_dialog);
	
	// Perform a padlock PCR based search
	int exact_match;
	
	char *buffer;
	
	XtVaGetValues(padlock_dialog_primer1_text, XmNvalue, &buffer, NULL);
	padlock_primer1_text = buffer;
	
	XtVaGetValues(padlock_dialog_primer2_text, XmNvalue, &buffer, NULL);
	padlock_primer2_text = buffer;
	
	XtVaGetValues(padlock_dialog_strand_text, XmNvalue, &buffer, NULL);
	padlock_strand_text = buffer;
	
	XtVaGetValues(padlock_dialog_salt_text, XmNvalue, &buffer, NULL);
	padlock_salt_text = buffer;
	
	XtVaGetValues(padlock_dialog_tm_text, XmNvalue, &buffer, NULL);
	padlock_tm_text = buffer;
		
	XtVaGetValues(padlock_dialog_exact_text, XmNvalue, &buffer, NULL);
	padlock_exact_text = buffer;
	
	exact_match = atoi(buffer);
	
	if(exact_match < 0){
		ArrowCursor(padlock_dialog);
		
		AfxMessageBox("Please enter 0 or more bases for the ligation site exact match");
		
		is_working = false;
		
		return;
	}
	
	try{
				
		padlock_results_text = ptrGP->search_MOL_primers(
			padlock_primer1_text, 
			padlock_primer2_text, 
			atof( padlock_salt_text.c_str() ), 
			atof( padlock_strand_text.c_str() ), 
			atof( padlock_tm_text.c_str() ),
			exact_match
			);
	}
	catch(const char *error){
		padlock_results_text = error;
	}
	
	// Display the results
	XtVaSetValues(padlock_dialog_results_text, XmNvalue, 
		(char*)padlock_results_text.c_str(), NULL);
	
	ArrowCursor(padlock_dialog);
	
	// Update the display to highlight any selected annotations
	postRedisplay();
	
	is_working = false;
}

void PadlockDialogShowNext(Widget w, XtPointer data, XtPointer callData)
{
	FindDialogSearchAgain(w, data, callData);
}

void PadlockDialogAnnotate(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":PadlockDialogAnnotate: NULL ptrGP";
	}
	
	bool ret = ptrGP->annotate_selection();
	
	if(ret == false){
		AfxMessageBox("There must be at least one selected sequence region to annotate");
	}
	else{
		// Show the new annotation(s)
		postRedisplay();
	}
}

void PadlockDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(padlock_dialog);
	
	// Allow the user to open another dialog at a later date
	padlock_dialog = NULL;
}

void OptionsDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one options dialog is allowed at any one time.
	if(options_dialog){
		return;
	}
	
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialog: NULL ptrGP";
	}
	
	Widget pane, form, scale, button;
	
	options_dialog = XtVaCreatePopupShell("options", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(options_dialog, 
		XmNtitle, "Options", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(options_dialog, OptionsDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		options_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 6,
		XmNwidth, 600, 
		XmNheight, 200, 
		NULL);
	
	// Create static text labels
	XtVaCreateManagedWidget("Font scaling: ",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	XtVaCreateManagedWidget("Base content window size: ",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	XtVaCreateManagedWidget("Tm window size: ",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	XtVaCreateManagedWidget("Tm [Na+] (M): ",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	XtVaCreateManagedWidget("Tm [Strand] (M): ",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	// The variable strings
	std::stringstream sout;
	
	sout << ptrGP->font_scale();
	
	options_dialog_font_scaling = 
		XtVaCreateManagedWidget(sout.str().c_str(), 
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	sout.str("");
	sout << ptrGP->content_window();
	
	options_dialog_gc_window = 
		XtVaCreateManagedWidget(sout.str().c_str(),
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	sout.str("");
	sout << ptrGP->tm_window();
	
	options_dialog_tm_window = 
		XtVaCreateManagedWidget(sout.str().c_str(),
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	sout.str("");
	sout << ptrGP->tm_salt();
		
	options_dialog_salt =
		XtVaCreateManagedWidget(sout.str().c_str(),
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	sout.str("");
	sout << ptrGP->tm_strand();
	
	options_dialog_strand =
		XtVaCreateManagedWidget(sout.str().c_str(),
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
		
	// The slider bars
	scale = XtVaCreateManagedWidget("scale_font", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmHORIZONTAL,
		XmNminimum, ptrGP->min_font_scale_index(),
		XmNmaximum, ptrGP->max_font_scale_index(),
		XmNshowValue, False,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, ptrGP->font_scale_index(), NULL);
	XtAddCallback(scale, XmNdragCallback, OptionsDialogFontScale, NULL);
	
	scale = XtVaCreateManagedWidget("scale_gc", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmHORIZONTAL,
		XmNminimum, ptrGP->min_content_window(),
		XmNmaximum, ptrGP->max_content_window(),
		XmNshowValue, False,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, ptrGP->content_window(), NULL);
	XtAddCallback(scale, XmNdragCallback, OptionsDialogGCWindow, NULL);
	
	scale = XtVaCreateManagedWidget("scale_tm", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmHORIZONTAL,
		XmNminimum, ptrGP->min_tm_window(),
		XmNmaximum, ptrGP->max_tm_window(),
		XmNshowValue, False,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, ptrGP->tm_window(), NULL);
	XtAddCallback(scale, XmNdragCallback, OptionsDialogTmWindow, NULL);
	
	scale = XtVaCreateManagedWidget("scale_salt", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmHORIZONTAL,
		XmNminimum, ptrGP->min_tm_salt_index(),
		XmNmaximum, ptrGP->max_tm_salt_index(),
		XmNshowValue, False,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, ptrGP->tm_salt_index(), NULL);
	XtAddCallback(scale, XmNdragCallback, OptionsDialogSalt, NULL);
	
	scale = XtVaCreateManagedWidget("scale_strand", 
		xmScaleWidgetClass, form, 
		XmNorientation, XmHORIZONTAL,
		XmNminimum, ptrGP->min_tm_strand_index(),
		XmNmaximum, ptrGP->max_tm_strand_index(),
		XmNshowValue, False,
		//XmNsensitive, False,
		// Don't use the scale title! It appears to be greyed out!
		//XtVaTypedArg, XmNtitleString, XmRString, title, strlen(title),
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaSetValues(scale, XmNvalue, ptrGP->tm_strand_index(), NULL);
	XtAddCallback(scale, XmNdragCallback, OptionsDialogStrand, NULL);
	
	// The close button
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 5,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 6,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		OptionsDialogClose, options_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(options_dialog, XtGrabNone);
}

void OptionsDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(options_dialog);
	
	// Allow the user to open another dialog at a later date
	options_dialog = NULL;
}
		 
void OptionsDialogFontScale(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialogFontScale: NULL ptrGP";
	}
	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	std::stringstream sout;
	
	ptrGP->font_scale_index(scale_data->value);
	
	sout << ptrGP->font_scale();
	
	XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
	XtVaSetValues(options_dialog_font_scaling, XmNlabelString, str, NULL);
	
	XmStringFree(str);
	
	postRedisplay();
}

void OptionsDialogGCWindow(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialogGCWindow: NULL ptrGP";
	}
	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	std::stringstream sout;
	
	ptrGP->content_window(scale_data->value);
	
	sout << ptrGP->content_window();
	
	XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
	XtVaSetValues(options_dialog_gc_window, XmNlabelString, str, NULL);
	
	XmStringFree(str);
	
	postRedisplay();
}

void OptionsDialogTmWindow(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialogTmWindow: NULL ptrGP";
	}
	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	std::stringstream sout;
	
	ptrGP->tm_window(scale_data->value);
	
	sout << ptrGP->tm_window();
	
	XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
	XtVaSetValues(options_dialog_tm_window, XmNlabelString, str, NULL);
	
	XmStringFree(str);
	
	postRedisplay();
}

void OptionsDialogSalt(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialogSalt: NULL ptrGP";
	}
	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	std::stringstream sout;
	
	ptrGP->tm_salt_index(scale_data->value);
	
	sout << ptrGP->tm_salt();
	
	XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
	XtVaSetValues(options_dialog_salt, XmNlabelString, str, NULL);
	
	XmStringFree(str);
	
	postRedisplay();
}

void OptionsDialogStrand(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":OptionsDialogStrand: NULL ptrGP";
	}
	
	XmScaleCallbackStruct *scale_data = (XmScaleCallbackStruct*)callData;
	std::stringstream sout;
	
	ptrGP->tm_strand_index(scale_data->value);
	
	sout << ptrGP->tm_strand();
	
	XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
	XtVaSetValues(options_dialog_strand, XmNlabelString, str, NULL);
	
	XmStringFree(str);
	
	postRedisplay();
}

void DownloadDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one download dialog is allowed at any one time.
	if(download_dialog){
		return;
	}
		
	Widget pane, form, button, option;
	
	download_dialog = XtVaCreatePopupShell("Download", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(download_dialog, 
		XmNtitle, "Download (NCBI)", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(download_dialog, DownloadDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		download_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 9,
		//XmNwidth, 450, 
		//XmNheight, 300, 
		NULL);
	
	// Create static text label.
	XtVaCreateManagedWidget("Search by:",xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_END,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		NULL);
		
	// Add the option menu
	XmString op_accession =	XmStringCreateSimple("Accession");
	XmString op_gi =		XmStringCreateSimple("GI");
	XmString op_taxa = 		XmStringCreateSimple("Taxa Name");
	XmString op_user = 		XmStringCreateSimple("User Defined (Entrez)");
	
	option = XmVaCreateSimpleOptionMenu(form, "option_menu", 
		NULL, 'N' /* Default key */, download_key,
		DownloadDialogKey /* The Callback function */,
		XmVaPUSHBUTTON, op_accession,'A', NULL, NULL, 
		XmVaPUSHBUTTON, op_gi, 'G', NULL, NULL, 
		XmVaPUSHBUTTON, op_taxa, 'T', NULL, NULL, 
		XmVaPUSHBUTTON, op_user, 'U', NULL, NULL, 
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	XmStringFree(op_accession);
	XmStringFree(op_gi);
	XmStringFree(op_taxa);
	XmStringFree(op_user);
	
	XtManageChild(option);
	
	Arg arg_list[14];
	
	XtSetArg(arg_list[0], XmNselectionPolicy, XmEXTENDED_SELECT);
	XtSetArg(arg_list[1], XmNalignment, XmALIGNMENT_BEGINNING);
	XtSetArg(arg_list[2], XmNtopAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[3], XmNtopPosition, 2);
	XtSetArg(arg_list[4], XmNbottomAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[5], XmNbottomPosition, 8);
	XtSetArg(arg_list[6], XmNleftAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[7], XmNleftPosition, 0);
	XtSetArg(arg_list[8], XmNrightAttachment, XmATTACH_POSITION);
	XtSetArg(arg_list[9], XmNrightPosition, 9);
	XtSetArg(arg_list[10], XmNvisibleItemCount, 10);
	XtSetArg(arg_list[11], XmNitemCount, 0);
	XtSetArg(arg_list[12], XmNitems, NULL);
	XtSetArg(arg_list[13], XmNlistSizePolicy, XmCONSTANT);
	
	// Add the scrollable list to display search results.
	download_dialog_doc_list = XmCreateScrolledList(form, "doc", arg_list, 14);
						   
	XtManageChild(download_dialog_doc_list);
	    
	XtAddCallback (download_dialog_doc_list, 
		XmNdefaultActionCallback, DownloadDialogListCallback, NULL);
	
	XtAddCallback (download_dialog_doc_list, 
		XmNmultipleSelectionCallback, DownloadDialogListCallback, NULL);
	    
	download_dialog_search_text = XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)download_search_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	button = XtVaCreateManagedWidget("Search",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		DownloadDialogSearch, download_dialog);
		
	button = XtVaCreateManagedWidget("Load",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		DownloadDialogLoad, download_dialog);
	
	button = XtVaCreateManagedWidget("Load & Save",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		DownloadDialogLoadAndSave, download_dialog);
		
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
	
	XtAddCallback(button, XmNactivateCallback, 
		DownloadDialogClose, download_dialog);
		
	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(download_dialog, XtGrabNone);

}

void DownloadDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(download_dialog);
	
	// Allow the user to open another dialog at a later date
	download_dialog = NULL;
}

void DownloadDialogListCallback(Widget w, XtPointer data, XtPointer callData)
{
	// Do nothing!
}

void DownloadDialogKey(Widget w, XtPointer data, XtPointer callData)
{
	char *type_name = XtName(w);
	
	// Note that XtName(widget) returns a normal text string
	// "button_0", "button_1", "button_2" etc, depending on the
	// item selected.
	if(strcmp(type_name, "button_0") == 0){
		download_key = ENTREZ_GENOME_ACCESSION;
		return;
	}
	
	if(strcmp(type_name, "button_1") == 0){
		download_key = ENTREZ_GENOME_GI;
		return;
	}
	
	if(strcmp(type_name, "button_2") == 0){
		download_key = ENTREZ_GENOME_TAXA_NAME;
		return;
	}
	
	if(strcmp(type_name, "button_3") == 0){
		download_key = ENTREZ_GENOME_USER_DEFINED;
		return;
	}
}

void DownloadDialogSearch(Widget w, XtPointer data, XtPointer callData)
{	
	// Only one search at a time
	static bool is_searching = false;
	
	if(is_searching){
		return;
	}
	
	is_searching = true;
	
	WaitCursor(download_dialog);
	
	char *buffer;
	
	// Load the search key and search string from the dialog and
	// start searching
	
	// Don't free this memory!
	XtVaGetValues(download_dialog_search_text, XmNvalue, &buffer, NULL);
	
	// Save the search string for persistance
	download_search_text = buffer;
	
	switch(download_key){
		case ENTREZ_GENOME_ACCESSION:
		case ENTREZ_GENOME_GI:
		case ENTREZ_GENOME_TAXA_NAME:
		case ENTREZ_GENOME_USER_DEFINED:
			docsum_list = docsum_list_from_search_key(buffer, download_key);
			break;
		default:
			AfxMessageBox("Unknown search key!");
			break;
	}
	
	// Flush the existing list items
	XmListDeleteAllItems(download_dialog_doc_list);
	
	// Populate the list
	std::list< std::pair<unsigned int, std::string> >::const_iterator iter;
	int pos = 1;
	
	for(iter = docsum_list.begin();iter != docsum_list.end();iter++, pos++){
		stringstream sout;
		
		sout << "(" << iter->first << ") " << iter->second;
		
		XmString str = XmStringCreateLocalized((char*)(sout.str().c_str()));
	
		XmListAddItem(download_dialog_doc_list, str, pos);
		
		XmStringFree(str);
	}
	 
	ArrowCursor(download_dialog);
				
	is_searching = false;
}

void DownloadDialogLoad(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":DownloadDialogLoad: NULL ptrGP";
	}
	
	string file_name;
	bool save_file = false;
	
	// When performing a load and save, data == NULL.
	// When performing a load, data == download_dialog
	if(data == NULL){
	
		// Copy the text from the file selection dialog to our
		// text widget.
		char *tmp_str = NULL;
	
		XmFileSelectionBoxCallbackStruct *ptrFileSel = 
			(XmFileSelectionBoxCallbackStruct *)callData;

  		XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);

  		file_name = tmp_str;
		save_file = true;
		
  		if(tmp_str){
			XtFree(tmp_str);
		}

		// We don't really need to destroy the widget! We should probably
		// just un-map it (since we allready keep a static pointer to it).
		XtDestroyWidget(file_dialog);

		// Allow the user to open another browse dialog at a later date
		file_dialog = NULL;
	}
	
	static bool is_loading = false;
	
	if(is_loading){
		return;
	}
	
	is_loading = true;
	
	WaitCursor(download_dialog);
	
	// Get all of the selected list entries
	int *pos_list = NULL;
	int pos_count = 0;
	
	Boolean ret = XmListGetSelectedPos(download_dialog_doc_list, &pos_list, &pos_count);
		
	if((ret == FALSE) || (pos_count == 0)){
	
		ArrowCursor(download_dialog);
		
		AfxMessageBox("Please specify a valid search result to load");
		
		is_loading = false;
		
		return;
	}
	
	if(save_file && (pos_count != 1)){
		AfxMessageBox("Please select one and only one entry to save");
		is_loading = false;
		return;
	}
	
	for(int pos = 0;pos < pos_count;pos++){
	
		// Locations in the list are 1's based
		int count = 1;
		std::list< std::pair<unsigned int, std::string> >::const_iterator iter = 
			docsum_list.begin();
			
		while(count < pos_list[pos]){
		
			count++;
			iter++;
			
			if(iter == docsum_list.end()){
				ArrowCursor(download_dialog);
				
				AfxMessageBox("Error reading search list. Please contact " 
					EMAIL_ADDRESS);
				
				return;
			}
		}
				
		try{
			bool ret = true;
			
			if(save_file){
				ret = ptrGP->load_and_save(iter->first, file_name);
			}
			else{
				ret = ptrGP->load(iter->first);
			}
			
			if(ret == false){
			
				AfxMessageBox("Did not read any sequence data for this genome "
					"Try downloading this file directly from NCBI");
			}
		}
		catch(const char *error){
			ArrowCursor(download_dialog);
			AfxMessageBox(error);
			
			is_loading = false;
			
			return;
		}
		catch(...){
			ArrowCursor(download_dialog);
			AfxMessageBox("An unhandled error has occured! "
				"Please report to " EMAIL_ADDRESS);
			
			is_loading = false;
			
			return;
		}
	}	
	
	if(pos_list != NULL){
		XtFree((char*)pos_list);
	}
	
	ArrowCursor(download_dialog);
	
	// Update the display
	postRedisplay();
	
	is_loading = false;
}

void DownloadDialogLoadAndSave(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":DownloadDialogLoadAndSave: NULL ptrGP";
	}
	
	// Open a file dialog to prompt the user for a file name
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*.asn", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, DownloadDialogLoad, NULL);
		
		// No need for a separate OnFileExportCancel -- just need to close the
		// dialog box
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void WaitCursor(Widget w)
{
	Cursor wait_cursor = XCreateFontCursor(XtDisplay(w), XC_watch);
	
	XDefineCursor(XtDisplay(w), XtWindow(w), wait_cursor);
	XmUpdateDisplay(w);
}

void ArrowCursor(Widget w)
{
	XUndefineCursor(XtDisplay(w), XtWindow(w));
	XmUpdateDisplay(w);
}

void PrimerDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one primer dialog is allowed at any one time.
	if(primer_dialog){
		return;
	}
		
	Widget pane, form, button;
	
	primer_dialog = XtVaCreatePopupShell("Primer_Prediction", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(primer_dialog, 
		XmNtitle, "Primer Prediction", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(primer_dialog, PrimerDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		primer_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 10,
		XmNwidth, 450, 
		XmNheight, 350, 
		NULL);
		
	// Create static text labels
	XtVaCreateManagedWidget("Primer length =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
		NULL);
	
	primer_dialog_primer_length_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_len_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
		
	XtVaCreateManagedWidget("bases",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	XtVaCreateManagedWidget("Minimium primer melting temperature = ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	primer_dialog_min_tm_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_tm_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 6,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 8,
		NULL);
		
	XtVaCreateManagedWidget("C",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 8,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
	
	XtVaCreateManagedWidget("Maximum hairpin/dimer melting temperature = ",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		NULL);
	
	primer_dialog_hairpin_tm_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_hairpin_tm_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
		
	XtVaCreateManagedWidget("C",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 9,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
	
	XtVaCreateManagedWidget("[Na+] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 2,
		NULL);
	
	primer_dialog_salt_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_salt_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		NULL);
		
	XtVaCreateManagedWidget("M, [Strand] =",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 4,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 6,
		NULL);
	
	primer_dialog_strand_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_strand_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 6,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 8,
		NULL);
		
	XtVaCreateManagedWidget("M",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 8,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
		
  	primer_dialog_3_GC_toggle = XtVaCreateManagedWidget("Avoid 3 or more G's or C's at 3' terminal end",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
    		NULL);
	
	XmToggleButtonSetState(primer_dialog_3_GC_toggle, 
		primer_3_GC_toggle, False);
		
	primer_dialog_GC_5_toggle = XtVaCreateManagedWidget("Require low %G+C at 5' terminal end",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 5,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 6,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
    		NULL);
	
	XmToggleButtonSetState(primer_dialog_GC_5_toggle, 
		primer_GC_5_toggle, False);
		
	primer_dialog_run_toggle = XtVaCreateManagedWidget("Avoid runs of",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 6,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 7,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 3,
    		NULL);
	
	XmToggleButtonSetState(primer_dialog_run_toggle, 
		primer_run_toggle, False);
	
	primer_dialog_run_length_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_run_len_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 6,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 7,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 3,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
	
	XtVaCreateManagedWidget("or more T/C or A/G bases",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 6,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 7,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		NULL);
		
	primer_dialog_GC_toggle = XtVaCreateManagedWidget("Require primer %G+C in the range",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 7,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 8,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(primer_dialog_GC_toggle, 
		primer_GC_toggle, False);
	
	primer_dialog_min_GC_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_min_gc_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 7,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 8,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 7,
		NULL);
	
	XtVaCreateManagedWidget("to",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 7,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 8,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 7,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 8,
		NULL);
		
	primer_dialog_max_GC_text = 
		XtVaCreateManagedWidget("",xmTextWidgetClass, form,
		XmNeditable, True,
		XmNvalue, (char*)primer_max_gc_text.c_str(),
		XmNeditMode, XmSINGLE_LINE_EDIT,
		XmNalignment, XmALIGNMENT_CENTER,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 7,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 8,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 8,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 10,
		NULL);
		
	primer_dialog_3_T_toggle = XtVaCreateManagedWidget("Avoid a T at the 3' terminal end",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
    		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
    		NULL);
	
	XmToggleButtonSetState(primer_dialog_3_T_toggle, 
		primer_3_T_toggle, False);
	
	button = XtVaCreateManagedWidget("Ok",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 9,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 2,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
		
	XtAddCallback(button, XmNactivateCallback, 
		PrimerDialogOk, primer_dialog);
	
	XtVaSetValues(form, XmNinitialFocus, button, NULL);
	
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 9,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 6,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 8,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
		
	XtAddCallback(button, XmNactivateCallback, 
		PrimerDialogClose, primer_dialog);

	XtManageChild(form);
	
	XtManageChild(pane);
	
	XtPopup(primer_dialog, XtGrabNone);
}

void PrimerDialogOk(Widget w, XtPointer data, XtPointer callData)
{
	static bool is_searching = false;
	
	// Update persistent dialog values
	if(is_searching){
		return;
	}
	
	is_searching = true;
	
	WaitCursor(primer_dialog);
	
	try{
	
		char *buffer;

		int len;
		float tm;
		float hairpin_tm;
		float salt;
		float strand;
		int run_len;
		float min_gc;
		float max_gc;
		
		if(ptrGP == NULL){
			throw __FILE__ ":PrimerDialogOk: NULL ptrGP";
		}

		// Extract and save dialog values
		XtVaGetValues(primer_dialog_primer_length_text, XmNvalue, &buffer, NULL);
		primer_len_text = buffer;
		len = (int)( atof(buffer) );
		
		XtVaGetValues(primer_dialog_min_tm_text, XmNvalue, &buffer, NULL);
		primer_tm_text = buffer;
		tm = atof(buffer);
		
		XtVaGetValues(primer_dialog_hairpin_tm_text, XmNvalue, &buffer, NULL);
		primer_hairpin_tm_text = buffer;
		hairpin_tm = atof(buffer);
		
		XtVaGetValues(primer_dialog_salt_text, XmNvalue, &buffer, NULL);
		primer_salt_text = buffer;
		salt = atof(buffer);
		
		XtVaGetValues(primer_dialog_strand_text, XmNvalue, &buffer, NULL);
		primer_strand_text = buffer;
		strand = atof(buffer);
		
		XtVaGetValues(primer_dialog_run_length_text, XmNvalue, &buffer, NULL);
		primer_run_len_text = buffer;
		run_len = (int)( atof(buffer) );
		
		XtVaGetValues(primer_dialog_min_GC_text, XmNvalue, &buffer, NULL);
		primer_min_gc_text = buffer;
		min_gc = atof(buffer);
		
		XtVaGetValues(primer_dialog_max_GC_text, XmNvalue, &buffer, NULL);
		primer_max_gc_text = buffer;
		max_gc = atof(buffer);
		
		unsigned int mask = 0;
		
		if( (primer_3_GC_toggle = XmToggleButtonGetState(primer_dialog_3_GC_toggle)) ){
			mask |= POLY_3_GC;
		}
		
		if( (primer_GC_5_toggle = XmToggleButtonGetState(primer_dialog_GC_5_toggle)) ){
			mask |= MULTI_5_GC;
		}
		
		if( (primer_run_toggle = XmToggleButtonGetState(primer_dialog_run_toggle)) ){
			mask |= NO_POLY_RUNS;
		}
		
		if( (primer_GC_toggle = XmToggleButtonGetState(primer_dialog_GC_toggle)) ){
			mask |= GC_CONTENT;
		}
		
		if( (primer_3_T_toggle = XmToggleButtonGetState(primer_dialog_3_T_toggle)) ){
			mask |= NO_3_T;
		}
		
		ptrGP->compute_primers(mask,
			len,
			run_len,
			salt,
			strand,
			min_gc,
			max_gc,
			tm,
			hairpin_tm);
		
		postRedisplay();
	}
	catch(const char *error){
		ArrowCursor(primer_dialog);
		AfxMessageBox(error);
		is_searching = false;
	}
	catch(...){
		ArrowCursor(primer_dialog);
		AfxMessageBox("Unknown error occured while picking primers");
		is_searching = false;
	}
	
	ArrowCursor(primer_dialog);
	
	is_searching = false;
}

void PrimerDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(primer_dialog);
	
	// Allow the user to open another dialog at a later date
	primer_dialog = NULL;
}

// This function (AddMenuItem) is based on the NEdit code in menu.c. 
// Thanks NEdit!
void AddMenuItem(Widget parent, char *name, char *label,
	char mnemonic, XtCallbackProc callback)
{
	Widget button;
	XmString tmp_string = XmStringCreateSimple(label);
	
	button = XtVaCreateWidget(name, xmPushButtonWidgetClass, parent, 
    	    XmNlabelString, tmp_string, XmNmnemonic, mnemonic, NULL);
	    
    XtAddCallback(button, XmNactivateCallback, callback, NULL);
    
    XmStringFree(tmp_string);
    
    XtManageChild(button);
}

void AddMenuSeperator(Widget parent)
{
	Widget button = XmCreateSeparator(parent, "seperator", NULL, 0);
     XtManageChild(button);
}

void PasteClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params)
{
	XmTextPaste (w);
}	
	
void CopyClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params)
{
	XmTextCopy (w, event->xkey.time);
}	

void CutClipboard(Widget w, XEvent *event, String *args, Cardinal *num_params)
{
	XmTextCut (w, event->xkey.time);
}

void OnHScroll(Widget w, XtPointer data, XtPointer callData)
{
	if(NULL == ptrGP){
		throw __FILE__ ":OnHScroll: NULL ptrGP";
	}
	
	int slider_loc;
	
	XtVaGetValues(w, XmNvalue, &slider_loc, NULL);
	
	if(ptrGP->scroll_bar(slider_loc, HSCROLL_MAX - HSCROLL_MIN) == true){
		postRedisplay();
	}
	
	// Always update the current base position
	UpdateStatusBar();
}

// After every scroll, the slider "snaps back" to the center of the scroll bar
void OnHScrollSnapBack(Widget w, XtPointer data, XtPointer callData)
{
	if(NULL == ptrGP){
		throw __FILE__ ":OnHScrollSnapBack: NULL ptrGP";
	}
	
	XtVaSetValues(w, XmNvalue, HSCROLL_CENTER, NULL);
	
	// Reset the scroll bar position
	ptrGP->scroll_bar(HSCROLL_CENTER);
}

// After every scroll, the slider "snaps back" to the center of the scroll bar
void OnHScrollAndSnap(Widget w, XtPointer data, XtPointer callData)
{	
	if(NULL == ptrGP){
		throw __FILE__ ":OnHScrollAndSnap: NULL ptrGP";
	}
	
	int slider_loc;
	
	XtVaGetValues(w, XmNvalue, &slider_loc, NULL);
	
	if(ptrGP->scroll_bar(slider_loc, HSCROLL_MAX - HSCROLL_MIN) == true){
		postRedisplay();
	}
	
	// Reset the scroll bar position
	ptrGP->scroll_bar(HSCROLL_CENTER);
	
	XtVaSetValues(w, XmNvalue, HSCROLL_CENTER, NULL);
	
	// Always update the current base position
	UpdateStatusBar();
}

void OnVScroll(Widget w, XtPointer data, XtPointer callData)
{
	if(NULL == ptrGP){
		throw __FILE__ ":OnVScroll: NULL ptrGP";
	}
	
	int slider_loc;
	
	XtVaGetValues(w, XmNvalue, &slider_loc, NULL);
	
	if(ptrGP->zoom_bar(slider_loc, VSCROLL_MAX - VSCROLL_MIN) == true){
		postRedisplay();
	}
	
	// Always update the current base position
	UpdateStatusBar();
}

// After every scroll, the slider "snaps back" to the center of the scroll bar
void OnVScrollSnapBack(Widget w, XtPointer data, XtPointer callData)
{
	if(NULL == ptrGP){
		throw __FILE__ ":OnVScrollSnapBack: NULL ptrGP";
	}
	
	XtVaSetValues(w, XmNvalue, VSCROLL_CENTER, NULL);
	
	// Reset the scroll bar position
	ptrGP->zoom_bar(VSCROLL_CENTER);
}

// After every scroll, the slider "snaps back" to the center of the scroll bar
void OnVScrollAndSnap(Widget w, XtPointer data, XtPointer callData)
{	
	if(NULL == ptrGP){
		throw __FILE__ ":OnVScrollAndSnap: NULL ptrGP";
	}
	
	int slider_loc;
	
	XtVaGetValues(w, XmNvalue, &slider_loc, NULL);
	
	if(ptrGP->zoom_bar(slider_loc, VSCROLL_MAX - VSCROLL_MIN) == true){
		postRedisplay();
	}
	
	// Reset the zoom bar position
	ptrGP->zoom_bar(VSCROLL_CENTER);
	
	XtVaSetValues(w, XmNvalue, VSCROLL_CENTER, NULL);
	
	// Always update the current base position
	UpdateStatusBar();
}

void ExportGenomeDialog(Widget w, XtPointer data, XtPointer callData)
{
	// Only one export genome dialog is allowed at any one time.
	if(export_genome_dialog){
		return;
	}
	
	Widget pane, form, button, radio_group;
	
	export_genome_dialog = XtVaCreatePopupShell("Export_Genome", 
		xmDialogShellWidgetClass,
		GetTopShell(w), 
		XmNdeleteResponse, XmDESTROY, 
		NULL);
	
	// Add a title to the dialog
	XtVaSetValues(export_genome_dialog, 
		XmNtitle, "Export Genome(s)", 
		NULL);
	
	// Call the Close() function when the user presses the
	// window manager close button.
	HandleClose(export_genome_dialog, ExportGenomeDialogClose);
	
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass,
		export_genome_dialog, XmNsashWidth, 1, XmNsashHeight, 1,
		NULL);
		
	// ***************************************************************
	// Create the control area
	form = XtVaCreateWidget("controlform", xmFormWidgetClass, pane, 
		XmNfractionBase, 11,
		XmNwidth, 350, 
		XmNheight, 350, 
		NULL);
		
	XtVaCreateManagedWidget("Annotations",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
		NULL);
	
	export_genome_gene_toggle = 
		XtVaCreateManagedWidget("Gene",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 2,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_gene_toggle, 
		True, False);
	
	export_genome_pseudo_gene_toggle = 
		XtVaCreateManagedWidget("Pseudo-gene",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 2,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 3,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_pseudo_gene_toggle, 
		True, False);
		
	export_genome_CDS_toggle = 
		XtVaCreateManagedWidget("CDS",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 3,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 4,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_CDS_toggle, 
		True, False);
		
	export_genome_RNA_toggle = 
		XtVaCreateManagedWidget("RNA",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 4,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 5,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_RNA_toggle, 
		True, False);
		
	export_genome_tRNA_toggle = 
		XtVaCreateManagedWidget("tRNA",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 5,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 6,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_tRNA_toggle, 
		True, False);
		
	export_genome_misc_toggle = 
		XtVaCreateManagedWidget("Misc",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 6,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 7,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_misc_toggle, 
		True, False);
		
	export_genome_primer_toggle = 
		XtVaCreateManagedWidget("Primer",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 7,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 8,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_primer_toggle, 
		True, False);
		
	export_genome_user_toggle = 
		XtVaCreateManagedWidget("User defined",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 8,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 9,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_user_toggle, 
		True, False);
		
	export_genome_intergenic_toggle = 
		XtVaCreateManagedWidget("Intergenic space",
        	xmToggleButtonWidgetClass, form, 
    		XmNvisibleWhenOff, True,
		XmNindicatorOn, XmINDICATOR_CHECK_BOX,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 9,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 0,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 5,
    		NULL);
	
	XmToggleButtonSetState(export_genome_intergenic_toggle, 
		True, False);
		
	XtVaCreateManagedWidget("Formats",
		xmLabelWidgetClass, form,
		XmNalignment, XmALIGNMENT_BEGINNING,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 0,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 1,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 11,
		NULL);
	
	radio_group = XtVaCreateManagedWidget("radiogroup",
		xmRowColumnWidgetClass, form,
		XmNorientation, XmVERTICAL,
		XmNradioBehavior, True,
		XmNradioAlwaysOne, True,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 10,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 5,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 11,
		NULL);
	
	export_genome_fasta_genome_radio = 
		XtVaCreateManagedWidget("Fasta (entire genome)",
        	xmToggleButtonWidgetClass, radio_group, 
    		XmNvisibleWhenOff, True,
		XmNradioBehavior, True,
		XmNindicatorType, XmONE_OF_MANY_ROUND,
    		NULL);
	
	XtAddCallback(export_genome_fasta_genome_radio, XmNvalueChangedCallback, 
                  ExportGenomeToggleFastaGenome, NULL);
			   
	export_genome_fasta_dna_radio = 
		XtVaCreateManagedWidget("Fasta (DNA)",
        	xmToggleButtonWidgetClass, radio_group, 
    		XmNvisibleWhenOff, True,
		XmNradioBehavior, True,
		XmNindicatorType, XmONE_OF_MANY_ROUND,
    		NULL);
	
	XtAddCallback(export_genome_fasta_dna_radio, XmNvalueChangedCallback, 
                  ExportGenomeToggleFastaDNA, NULL);
			   
	export_genome_fasta_aa_radio = 
		XtVaCreateManagedWidget("Fasta (amino acid)",
       	xmToggleButtonWidgetClass, radio_group, 
    		XmNvisibleWhenOff, True,
		XmNradioBehavior, True,
		XmNindicatorType, XmONE_OF_MANY_ROUND,
    		NULL);
	
	XtAddCallback(export_genome_fasta_aa_radio, XmNvalueChangedCallback, 
                  ExportGenomeToggleFastaProtein, NULL);
			   
	export_genome_fasta_gbk_radio = 
		XtVaCreateManagedWidget("GBK (genbank annotation)",
        	xmToggleButtonWidgetClass, radio_group, 
    		XmNvisibleWhenOff, True,
		XmNradioBehavior, True,
		XmNindicatorType, XmONE_OF_MANY_ROUND,
    		NULL);
	
	XtAddCallback(export_genome_fasta_gbk_radio, XmNvalueChangedCallback, 
                  ExportGenomeToggleGBK, NULL);
			   		
	export_genome_fasta_ptt_radio = 
		XtVaCreateManagedWidget("PTT (protein table)",
        	xmToggleButtonWidgetClass, radio_group, 
    		XmNvisibleWhenOff, True,
		XmNradioBehavior, True,
		XmNindicatorType, XmONE_OF_MANY_ROUND,
    		NULL);
	
	XtAddCallback(export_genome_fasta_ptt_radio, XmNvalueChangedCallback, 
                  ExportGenomeTogglePTT, NULL);
			   
	button = XtVaCreateManagedWidget("Export",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 11,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 1,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 4,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
		
	XtAddCallback(button, XmNactivateCallback, 
		ExportGenomeDialogOk, export_genome_dialog);
	
	XtVaSetValues(form, XmNinitialFocus, button, NULL);
	
	button = XtVaCreateManagedWidget("Close",
		xmPushButtonGadgetClass, form,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, 10,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, 11,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, 6,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, 9,
		XmNshowAsDefault, True,
		XmNdefaultButtonShadowThickness, 1,
		NULL);
		
	XtAddCallback(button, XmNactivateCallback, 
		ExportGenomeDialogClose, export_genome_dialog);

	XtManageChild(radio_group);
	
	XtManageChild(form);
	
	XtManageChild(pane);
	
	// Make GBK our default export file format
	XmToggleButtonSetState(export_genome_fasta_gbk_radio, 
		True, True);
		
	XtPopup(export_genome_dialog, XtGrabNone);		
}

void ExportGenomeDialogOk(Widget w, XtPointer data, XtPointer callData)
{
	// Make sure that the user has at least one genome loaded
	if( (ptrGP == NULL) || (ptrGP->num_display() == 0) ){
		AfxMessageBox("Please open a genome before exporting");
		return;
	}
	
	// Clear any existing annotations
	ptrGP->clear_annotations_to_export();
	
	if(XmToggleButtonGetState(export_genome_gene_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::GENE);
	}
	
	if(XmToggleButtonGetState(export_genome_pseudo_gene_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::PSEUDO_GENE);
	}
	
	if(XmToggleButtonGetState(export_genome_CDS_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::CDS);
	}
	
	if(XmToggleButtonGetState(export_genome_RNA_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::RNA);
	}
	
	if(XmToggleButtonGetState(export_genome_tRNA_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::tRNA);
	}
	
	if(XmToggleButtonGetState(export_genome_misc_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::IMP);
	}
	
	if(XmToggleButtonGetState(export_genome_primer_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::PRIMER);
	}
	
	if(XmToggleButtonGetState(export_genome_user_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::USER);
	}
	
	if(XmToggleButtonGetState(export_genome_intergenic_toggle) == True){
		ptrGP->add_annotation_to_export(GeneAnnotation::NONE);
	}
	
	// Make sure that the user has selected at least one type of annotation
	// to export
	if(ptrGP->num_annotations_to_export() == 0){
	
		AfxMessageBox("Please specify at least one annotation type to export");
		return;
	}
	
	// Set the export format
	if(XmToggleButtonGetState(export_genome_fasta_genome_radio) == True){
	
		if(ptrGP->set_export_format(GenomePage::EXPORT_FASTA_GENOME) == false){
			AfxMessageBox("Please specify a valid export format");
			return;
		}
	}
	
	if(XmToggleButtonGetState(export_genome_fasta_dna_radio) == True){
	
		if(ptrGP->set_export_format(GenomePage::EXPORT_FASTA_DNA) == false){
			AfxMessageBox("Please specify a valid export format");
			return;
		}
	}
	
	if(XmToggleButtonGetState(export_genome_fasta_aa_radio) == True){
	
		if(ptrGP->set_export_format(GenomePage::EXPORT_FASTA_PROTEIN) == false){
			AfxMessageBox("Please specify a valid export format");
			return;
		}
	}
	
	if(XmToggleButtonGetState(export_genome_fasta_gbk_radio) == True){
	
		if(ptrGP->set_export_format(GenomePage::EXPORT_GBK) == false){
			AfxMessageBox("Please specify a valid export format");
			return;
		}
	}
	
	if(XmToggleButtonGetState(export_genome_fasta_ptt_radio) == True){
	
		if(ptrGP->set_export_format(GenomePage::EXPORT_PTT) == false){
			AfxMessageBox("The PTT format can only store a single genome");
			return;
		}
	}
	
	// Get a file name from the user
	// via the Motif file browser
	// Only allow one Browse dialog on the screen at one time
	if(!file_dialog){
		
  		Widget button;
  		Arg args[1];
		
	    	XtSetArg(args[0], XmNpattern,
    			XmStringCreate("*", XmSTRING_DEFAULT_CHARSET));
		
    		file_dialog = XmCreateFileSelectionDialog(XtParent(w), "filebox",
    			args, 1);
		
		// Pass the text sting to the "OK" call back function
   		XtAddCallback(file_dialog, XmNokCallback, ExportGenome, NULL);
		
		// No need for a separate OnFileExportCancel -- just need to close the
		// dialog box
   		XtAddCallback(file_dialog, XmNcancelCallback, OnFileOpenCancel, NULL);
    	
		/* Remove help button. */
    		button = XmFileSelectionBoxGetChild(file_dialog,
    			XmDIALOG_HELP_BUTTON);
    			XtUnmanageChild(button);
	}
	else{
		AfxMessageBox("Please close the file dialog box!");
	}
	
	XtManageChild(file_dialog);
	
	// Note! This handy function was gleened from the nice people who
	// wrote xmgr!
	XMapRaised(XtDisplay(file_dialog), XtWindow(file_dialog));
}

void ExportGenomeDialogClose(Widget w, XtPointer data, XtPointer callData)
{
	XtDestroyWidget(export_genome_dialog);
	
	// Allow the user to open another dialog at a later date
	export_genome_dialog = NULL;
}

void ExportGenomeToggleFastaGenome(Widget w, XtPointer data, XtPointer callData)
{
	XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)(callData);
	
	// Is the radio on or off?
	if(state->set != True){
		return;
	}
	
	// Turn on all annotation check boxes and deactivate them
	XmToggleButtonSetState(export_genome_gene_toggle, True, True);
	XmToggleButtonSetState(export_genome_pseudo_gene_toggle, True, True);
	XmToggleButtonSetState(export_genome_CDS_toggle, True, True);
	XmToggleButtonSetState(export_genome_RNA_toggle, True, True);
	XmToggleButtonSetState(export_genome_tRNA_toggle, True, True);
	XmToggleButtonSetState(export_genome_misc_toggle, True, True);
	XmToggleButtonSetState(export_genome_intergenic_toggle, True, True);
	XmToggleButtonSetState(export_genome_primer_toggle, True, True);
	XmToggleButtonSetState(export_genome_user_toggle, True, True);
	
	XtSetSensitive(export_genome_gene_toggle, False);
	XtSetSensitive(export_genome_pseudo_gene_toggle, False);
	XtSetSensitive(export_genome_CDS_toggle, False);
	XtSetSensitive(export_genome_RNA_toggle, False);
	XtSetSensitive(export_genome_tRNA_toggle, False);
	XtSetSensitive(export_genome_misc_toggle, False);
	XtSetSensitive(export_genome_intergenic_toggle, False);
	XtSetSensitive(export_genome_primer_toggle, False);
	XtSetSensitive(export_genome_user_toggle, False);
}

void ExportGenomeToggleFastaDNA(Widget w, XtPointer data, XtPointer callData)
{
	XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)(callData);
	
	// Is the radio on or off?
	if(state->set != True){
		return;
	}
	
	// Turn off intergenic space
	XmToggleButtonSetState(export_genome_intergenic_toggle, False, True);
	
	// Activate all annotation check boxes
	XtSetSensitive(export_genome_gene_toggle, True);
	XtSetSensitive(export_genome_pseudo_gene_toggle, True);
	XtSetSensitive(export_genome_CDS_toggle, True);
	XtSetSensitive(export_genome_RNA_toggle, True);
	XtSetSensitive(export_genome_tRNA_toggle, True);
	XtSetSensitive(export_genome_misc_toggle, True);
	XtSetSensitive(export_genome_intergenic_toggle, True);
	XtSetSensitive(export_genome_primer_toggle, True);
	XtSetSensitive(export_genome_user_toggle, True);
}

void ExportGenomeToggleFastaProtein(Widget w, XtPointer data, XtPointer callData)
{
	XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)(callData);
	
	// Is the radio on or off?
	if(state->set != True){
		return;
	}
		
	// Turn off selected annotation check boxes (i.e. non-amino
	// acid annotations). However, leave all annotations active
	XmToggleButtonSetState(export_genome_pseudo_gene_toggle, False, True);
	XmToggleButtonSetState(export_genome_RNA_toggle, False, True);
	XmToggleButtonSetState(export_genome_tRNA_toggle, False, True);
	XmToggleButtonSetState(export_genome_misc_toggle, False, True);
	XmToggleButtonSetState(export_genome_intergenic_toggle, False, True);
	XmToggleButtonSetState(export_genome_primer_toggle, False, True);
	XmToggleButtonSetState(export_genome_user_toggle, False, True);
	
	XtSetSensitive(export_genome_gene_toggle, True);
	XtSetSensitive(export_genome_pseudo_gene_toggle, True);
	XtSetSensitive(export_genome_CDS_toggle, True);
	XtSetSensitive(export_genome_RNA_toggle, True);
	XtSetSensitive(export_genome_tRNA_toggle, True);
	XtSetSensitive(export_genome_misc_toggle, True);
	XtSetSensitive(export_genome_intergenic_toggle, True);
	XtSetSensitive(export_genome_primer_toggle, True);
	XtSetSensitive(export_genome_user_toggle, True);
}

void ExportGenomeToggleGBK(Widget w, XtPointer data, XtPointer callData)
{
	XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)(callData);
	
	// Is the radio on or off?
	if(state->set != True){
		return;
	}
	
	// Turn off intergenic space
	XmToggleButtonSetState(export_genome_intergenic_toggle, False, True);
	
	// Make all of the annotation boxes sensitive
	XtSetSensitive(export_genome_gene_toggle, True);
	XtSetSensitive(export_genome_pseudo_gene_toggle, True);
	XtSetSensitive(export_genome_CDS_toggle, True);
	XtSetSensitive(export_genome_RNA_toggle, True);
	XtSetSensitive(export_genome_tRNA_toggle, True);
	XtSetSensitive(export_genome_misc_toggle, True);
	XtSetSensitive(export_genome_intergenic_toggle, True);
	XtSetSensitive(export_genome_primer_toggle, True);
	XtSetSensitive(export_genome_user_toggle, True);
}

void ExportGenomeTogglePTT(Widget w, XtPointer data, XtPointer callData)
{
	XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)(callData);
	
	// Is the radio on or off?
	if(state->set != True){
		return;
	}
	
	// Turn off intergenic space
	XmToggleButtonSetState(export_genome_intergenic_toggle, False, True);
	
	// Make all of the annotation boxes sensitive
	XtSetSensitive(export_genome_gene_toggle, True);
	XtSetSensitive(export_genome_pseudo_gene_toggle, True);
	XtSetSensitive(export_genome_CDS_toggle, True);
	XtSetSensitive(export_genome_RNA_toggle, True);
	XtSetSensitive(export_genome_tRNA_toggle, True);
	XtSetSensitive(export_genome_misc_toggle, True);
	XtSetSensitive(export_genome_intergenic_toggle, True);
	XtSetSensitive(export_genome_primer_toggle, True);
	XtSetSensitive(export_genome_user_toggle, True);
}

void ExportGenome(Widget w, XtPointer data, XtPointer callData)
{
	if(ptrGP == NULL){
		throw __FILE__ ":ExportGenome: Null ptrGP";
	}
	
	// Copy the text from the file selection dialog to our
	// text widget.
	char *tmp_str = NULL;
	XmFileSelectionBoxCallbackStruct *ptrFileSel = 
		(XmFileSelectionBoxCallbackStruct *)callData;
		
  	XmStringGetLtoR(ptrFileSel->value, XmSTRING_DEFAULT_CHARSET, &tmp_str);
  
  	string file_name = tmp_str;
	
  	if(tmp_str){
		XtFree(tmp_str);
	}
	
	WaitCursor(file_dialog);
	
	try{
		ptrGP->export_genomes(file_name);
	}
	catch(const char *error){
		AfxMessageBox(error);
	}
	catch(...){
		AfxMessageBox("An export error has occured");
	}
	
	ArrowCursor(file_dialog);
	
	// We don't really need to destroy the widget! We should probably
	// just un-map it (since we allready keep a static pointer to it).
	XtDestroyWidget(file_dialog);
	
	// Allow the user to open another browse dialog at a later date
	file_dialog = NULL;
}
